sqlglot.dialects.mysql API documentation

Edit on GitHub

# sqlglot.dialects.mysql

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    NormalizationStrategy,
      9    arrow_json_extract_sql,
     10    date_add_interval_sql,
     11    datestrtodate_sql,
     12    build_formatted_time,
     13    isnull_to_is_null,
     14    locate_to_strposition,
     15    max_or_greatest,
     16    min_or_least,
     17    no_ilike_sql,
     18    no_paren_current_date_sql,
     19    no_pivot_sql,
     20    no_tablesample_sql,
     21    no_trycast_sql,
     22    build_date_delta,
     23    build_date_delta_with_interval,
     24    rename_func,
     25    strposition_to_locate_sql,
     26)
     27from sqlglot.helper import seq_get
     28from sqlglot.tokens import TokenType
     29
     30
     31def _show_parser(*args: t.Any, **kwargs: t.Any) -> t.Callable[[MySQL.Parser], exp.Show]:
     32    def _parse(self: MySQL.Parser) -> exp.Show:
     33        return self._parse_show_mysql(*args, **kwargs)
     34
     35    return _parse
     36
     37
     38def _date_trunc_sql(self: MySQL.Generator, expression: exp.DateTrunc) -> str:
     39    expr = self.sql(expression, "this")
     40    unit = expression.text("unit").upper()
     41
     42    if unit == "WEEK":
     43        concat = f"CONCAT(YEAR({expr}), ' ', WEEK({expr}, 1), ' 1')"
     44        date_format = "%Y %u %w"
     45    elif unit == "MONTH":
     46        concat = f"CONCAT(YEAR({expr}), ' ', MONTH({expr}), ' 1')"
     47        date_format = "%Y %c %e"
     48    elif unit == "QUARTER":
     49        concat = f"CONCAT(YEAR({expr}), ' ', QUARTER({expr}) * 3 - 2, ' 1')"
     50        date_format = "%Y %c %e"
     51    elif unit == "YEAR":
     52        concat = f"CONCAT(YEAR({expr}), ' 1 1')"
     53        date_format = "%Y %c %e"
     54    else:
     55        if unit != "DAY":
     56            self.unsupported(f"Unexpected interval unit: {unit}")
     57        return self.func("DATE", expr)
     58
     59    return self.func("STR_TO_DATE", concat, f"'{date_format}'")
     60
     61
     62# All specifiers for time parts (as opposed to date parts)
     63# https://dev.mysql.com/doc/refman/8.0/en/date-and-time-functions.html#function_date-format
     64TIME_SPECIFIERS = {"f", "H", "h", "I", "i", "k", "l", "p", "r", "S", "s", "T"}
     65
     66
     67def _has_time_specifier(date_format: str) -> bool:
     68    i = 0
     69    length = len(date_format)
     70
     71    while i < length:
     72        if date_format[i] == "%":
     73            i += 1
     74            if i < length and date_format[i] in TIME_SPECIFIERS:
     75                return True
     76        i += 1
     77    return False
     78
     79
     80def _str_to_date(args: t.List) -> exp.StrToDate | exp.StrToTime:
     81    mysql_date_format = seq_get(args, 1)
     82    date_format = MySQL.format_time(mysql_date_format)
     83    this = seq_get(args, 0)
     84
     85    if mysql_date_format and _has_time_specifier(mysql_date_format.name):
     86        return exp.StrToTime(this=this, format=date_format)
     87
     88    return exp.StrToDate(this=this, format=date_format)
     89
     90
     91def _str_to_date_sql(
     92    self: MySQL.Generator, expression: exp.StrToDate | exp.StrToTime | exp.TsOrDsToDate
     93) -> str:
     94    return self.func("STR_TO_DATE", expression.this, self.format_time(expression))
     95
     96
     97def _trim_sql(self: MySQL.Generator, expression: exp.Trim) -> str:
     98    target = self.sql(expression, "this")
     99    trim_type = self.sql(expression, "position")
    100    remove_chars = self.sql(expression, "expression")
    101
    102    # Use TRIM/LTRIM/RTRIM syntax if the expression isn't mysql-specific
    103    if not remove_chars:
    104        return self.trim_sql(expression)
    105
    106    trim_type = f"{trim_type} " if trim_type else ""
    107    remove_chars = f"{remove_chars} " if remove_chars else ""
    108    from_part = "FROM " if trim_type or remove_chars else ""
    109    return f"TRIM({trim_type}{remove_chars}{from_part}{target})"
    110
    111
    112def _date_add_sql(
    113    kind: str,
    114) -> t.Callable[[MySQL.Generator, exp.Expression], str]:
    115    def func(self: MySQL.Generator, expression: exp.Expression) -> str:
    116        this = self.sql(expression, "this")
    117        unit = expression.text("unit").upper() or "DAY"
    118        return (
    119            f"DATE_{kind}({this}, {self.sql(exp.Interval(this=expression.expression, unit=unit))})"
    120        )
    121
    122    return func
    123
    124
    125def _ts_or_ds_to_date_sql(self: MySQL.Generator, expression: exp.TsOrDsToDate) -> str:
    126    time_format = expression.args.get("format")
    127    return _str_to_date_sql(self, expression) if time_format else self.func("DATE", expression.this)
    128
    129
    130def _remove_ts_or_ds_to_date(
    131    to_sql: t.Optional[t.Callable[[MySQL.Generator, exp.Expression], str]] = None,
    132    args: t.Tuple[str, ...] = ("this",),
    133) -> t.Callable[[MySQL.Generator, exp.Func], str]:
    134    def func(self: MySQL.Generator, expression: exp.Func) -> str:
    135        for arg_key in args:
    136            arg = expression.args.get(arg_key)
    137            if isinstance(arg, exp.TsOrDsToDate) and not arg.args.get("format"):
    138                expression.set(arg_key, arg.this)
    139
    140        return to_sql(self, expression) if to_sql else self.function_fallback_sql(expression)
    141
    142    return func
    143
    144
    145class MySQL(Dialect):
    146    # https://dev.mysql.com/doc/refman/8.0/en/identifiers.html
    147    IDENTIFIERS_CAN_START_WITH_DIGIT = True
    148
    149    # We default to treating all identifiers as case-sensitive, since it matches MySQL's
    150    # behavior on Linux systems. For MacOS and Windows systems, one can override this
    151    # setting by specifying `dialect="mysql, normalization_strategy = lowercase"`.
    152    #
    153    # See also https://dev.mysql.com/doc/refman/8.2/en/identifier-case-sensitivity.html
    154    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_SENSITIVE
    155
    156    TIME_FORMAT = "'%Y-%m-%d %T'"
    157    DPIPE_IS_STRING_CONCAT = False
    158    SUPPORTS_USER_DEFINED_TYPES = False
    159    SUPPORTS_SEMI_ANTI_JOIN = False
    160    SAFE_DIVISION = True
    161
    162    # https://prestodb.io/docs/current/functions/datetime.html#mysql-date-functions
    163    TIME_MAPPING = {
    164        "%M": "%B",
    165        "%c": "%-m",
    166        "%e": "%-d",
    167        "%h": "%I",
    168        "%i": "%M",
    169        "%s": "%S",
    170        "%u": "%W",
    171        "%k": "%-H",
    172        "%l": "%-I",
    173        "%T": "%H:%M:%S",
    174        "%W": "%a",
    175    }
    176
    177    class Tokenizer(tokens.Tokenizer):
    178        QUOTES = ["'", '"']
    179        COMMENTS = ["--", "#", ("/*", "*/")]
    180        IDENTIFIERS = ["`"]
    181        STRING_ESCAPES = ["'", '"', "\\"]
    182        BIT_STRINGS = [("b'", "'"), ("B'", "'"), ("0b", "")]
    183        HEX_STRINGS = [("x'", "'"), ("X'", "'"), ("0x", "")]
    184
    185        KEYWORDS = {
    186            **tokens.Tokenizer.KEYWORDS,
    187            "CHARSET": TokenType.CHARACTER_SET,
    188            "FORCE": TokenType.FORCE,
    189            "IGNORE": TokenType.IGNORE,
    190            "LOCK TABLES": TokenType.COMMAND,
    191            "LONGBLOB": TokenType.LONGBLOB,
    192            "LONGTEXT": TokenType.LONGTEXT,
    193            "MEDIUMBLOB": TokenType.MEDIUMBLOB,
    194            "TINYBLOB": TokenType.TINYBLOB,
    195            "TINYTEXT": TokenType.TINYTEXT,
    196            "MEDIUMTEXT": TokenType.MEDIUMTEXT,
    197            "MEDIUMINT": TokenType.MEDIUMINT,
    198            "MEMBER OF": TokenType.MEMBER_OF,
    199            "SEPARATOR": TokenType.SEPARATOR,
    200            "START": TokenType.BEGIN,
    201            "SIGNED": TokenType.BIGINT,
    202            "SIGNED INTEGER": TokenType.BIGINT,
    203            "UNLOCK TABLES": TokenType.COMMAND,
    204            "UNSIGNED": TokenType.UBIGINT,
    205            "UNSIGNED INTEGER": TokenType.UBIGINT,
    206            "YEAR": TokenType.YEAR,
    207            "_ARMSCII8": TokenType.INTRODUCER,
    208            "_ASCII": TokenType.INTRODUCER,
    209            "_BIG5": TokenType.INTRODUCER,
    210            "_BINARY": TokenType.INTRODUCER,
    211            "_CP1250": TokenType.INTRODUCER,
    212            "_CP1251": TokenType.INTRODUCER,
    213            "_CP1256": TokenType.INTRODUCER,
    214            "_CP1257": TokenType.INTRODUCER,
    215            "_CP850": TokenType.INTRODUCER,
    216            "_CP852": TokenType.INTRODUCER,
    217            "_CP866": TokenType.INTRODUCER,
    218            "_CP932": TokenType.INTRODUCER,
    219            "_DEC8": TokenType.INTRODUCER,
    220            "_EUCJPMS": TokenType.INTRODUCER,
    221            "_EUCKR": TokenType.INTRODUCER,
    222            "_GB18030": TokenType.INTRODUCER,
    223            "_GB2312": TokenType.INTRODUCER,
    224            "_GBK": TokenType.INTRODUCER,
    225            "_GEOSTD8": TokenType.INTRODUCER,
    226            "_GREEK": TokenType.INTRODUCER,
    227            "_HEBREW": TokenType.INTRODUCER,
    228            "_HP8": TokenType.INTRODUCER,
    229            "_KEYBCS2": TokenType.INTRODUCER,
    230            "_KOI8R": TokenType.INTRODUCER,
    231            "_KOI8U": TokenType.INTRODUCER,
    232            "_LATIN1": TokenType.INTRODUCER,
    233            "_LATIN2": TokenType.INTRODUCER,
    234            "_LATIN5": TokenType.INTRODUCER,
    235            "_LATIN7": TokenType.INTRODUCER,
    236            "_MACCE": TokenType.INTRODUCER,
    237            "_MACROMAN": TokenType.INTRODUCER,
    238            "_SJIS": TokenType.INTRODUCER,
    239            "_SWE7": TokenType.INTRODUCER,
    240            "_TIS620": TokenType.INTRODUCER,
    241            "_UCS2": TokenType.INTRODUCER,
    242            "_UJIS": TokenType.INTRODUCER,
    243            # https://dev.mysql.com/doc/refman/8.0/en/string-literals.html
    244            "_UTF8": TokenType.INTRODUCER,
    245            "_UTF16": TokenType.INTRODUCER,
    246            "_UTF16LE": TokenType.INTRODUCER,
    247            "_UTF32": TokenType.INTRODUCER,
    248            "_UTF8MB3": TokenType.INTRODUCER,
    249            "_UTF8MB4": TokenType.INTRODUCER,
    250            "@@": TokenType.SESSION_PARAMETER,
    251        }
    252
    253        COMMANDS = tokens.Tokenizer.COMMANDS - {TokenType.SHOW}
    254
    255    class Parser(parser.Parser):
    256        FUNC_TOKENS = {
    257            *parser.Parser.FUNC_TOKENS,
    258            TokenType.DATABASE,
    259            TokenType.SCHEMA,
    260            TokenType.VALUES,
    261        }
    262
    263        CONJUNCTION = {
    264            **parser.Parser.CONJUNCTION,
    265            TokenType.DAMP: exp.And,
    266            TokenType.XOR: exp.Xor,
    267            TokenType.DPIPE: exp.Or,
    268        }
    269
    270        TABLE_ALIAS_TOKENS = (
    271            parser.Parser.TABLE_ALIAS_TOKENS - parser.Parser.TABLE_INDEX_HINT_TOKENS
    272        )
    273
    274        RANGE_PARSERS = {
    275            **parser.Parser.RANGE_PARSERS,
    276            TokenType.MEMBER_OF: lambda self, this: self.expression(
    277                exp.JSONArrayContains,
    278                this=this,
    279                expression=self._parse_wrapped(self._parse_expression),
    280            ),
    281        }
    282
    283        FUNCTIONS = {
    284            **parser.Parser.FUNCTIONS,
    285            "DATE": lambda args: exp.TsOrDsToDate(this=seq_get(args, 0)),
    286            "DATE_ADD": build_date_delta_with_interval(exp.DateAdd),
    287            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "mysql"),
    288            "DATE_SUB": build_date_delta_with_interval(exp.DateSub),
    289            "DAY": lambda args: exp.Day(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    290            "DAYOFMONTH": lambda args: exp.DayOfMonth(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    291            "DAYOFWEEK": lambda args: exp.DayOfWeek(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    292            "DAYOFYEAR": lambda args: exp.DayOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    293            "INSTR": lambda args: exp.StrPosition(substr=seq_get(args, 1), this=seq_get(args, 0)),
    294            "ISNULL": isnull_to_is_null,
    295            "LOCATE": locate_to_strposition,
    296            "MAKETIME": exp.TimeFromParts.from_arg_list,
    297            "MONTH": lambda args: exp.Month(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    298            "MONTHNAME": lambda args: exp.TimeToStr(
    299                this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    300                format=exp.Literal.string("%B"),
    301            ),
    302            "STR_TO_DATE": _str_to_date,
    303            "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
    304            "TO_DAYS": lambda args: exp.paren(
    305                exp.DateDiff(
    306                    this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    307                    expression=exp.TsOrDsToDate(this=exp.Literal.string("0000-01-01")),
    308                    unit=exp.var("DAY"),
    309                )
    310                + 1
    311            ),
    312            "WEEK": lambda args: exp.Week(
    313                this=exp.TsOrDsToDate(this=seq_get(args, 0)), mode=seq_get(args, 1)
    314            ),
    315            "WEEKOFYEAR": lambda args: exp.WeekOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    316            "YEAR": lambda args: exp.Year(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    317        }
    318
    319        FUNCTION_PARSERS = {
    320            **parser.Parser.FUNCTION_PARSERS,
    321            "CHAR": lambda self: self._parse_chr(),
    322            "GROUP_CONCAT": lambda self: self.expression(
    323                exp.GroupConcat,
    324                this=self._parse_lambda(),
    325                separator=self._match(TokenType.SEPARATOR) and self._parse_field(),
    326            ),
    327            # https://dev.mysql.com/doc/refman/5.7/en/miscellaneous-functions.html#function_values
    328            "VALUES": lambda self: self.expression(
    329                exp.Anonymous, this="VALUES", expressions=[self._parse_id_var()]
    330            ),
    331        }
    332
    333        STATEMENT_PARSERS = {
    334            **parser.Parser.STATEMENT_PARSERS,
    335            TokenType.SHOW: lambda self: self._parse_show(),
    336        }
    337
    338        SHOW_PARSERS = {
    339            "BINARY LOGS": _show_parser("BINARY LOGS"),
    340            "MASTER LOGS": _show_parser("BINARY LOGS"),
    341            "BINLOG EVENTS": _show_parser("BINLOG EVENTS"),
    342            "CHARACTER SET": _show_parser("CHARACTER SET"),
    343            "CHARSET": _show_parser("CHARACTER SET"),
    344            "COLLATION": _show_parser("COLLATION"),
    345            "FULL COLUMNS": _show_parser("COLUMNS", target="FROM", full=True),
    346            "COLUMNS": _show_parser("COLUMNS", target="FROM"),
    347            "CREATE DATABASE": _show_parser("CREATE DATABASE", target=True),
    348            "CREATE EVENT": _show_parser("CREATE EVENT", target=True),
    349            "CREATE FUNCTION": _show_parser("CREATE FUNCTION", target=True),
    350            "CREATE PROCEDURE": _show_parser("CREATE PROCEDURE", target=True),
    351            "CREATE TABLE": _show_parser("CREATE TABLE", target=True),
    352            "CREATE TRIGGER": _show_parser("CREATE TRIGGER", target=True),
    353            "CREATE VIEW": _show_parser("CREATE VIEW", target=True),
    354            "DATABASES": _show_parser("DATABASES"),
    355            "SCHEMAS": _show_parser("DATABASES"),
    356            "ENGINE": _show_parser("ENGINE", target=True),
    357            "STORAGE ENGINES": _show_parser("ENGINES"),
    358            "ENGINES": _show_parser("ENGINES"),
    359            "ERRORS": _show_parser("ERRORS"),
    360            "EVENTS": _show_parser("EVENTS"),
    361            "FUNCTION CODE": _show_parser("FUNCTION CODE", target=True),
    362            "FUNCTION STATUS": _show_parser("FUNCTION STATUS"),
    363            "GRANTS": _show_parser("GRANTS", target="FOR"),
    364            "INDEX": _show_parser("INDEX", target="FROM"),
    365            "MASTER STATUS": _show_parser("MASTER STATUS"),
    366            "OPEN TABLES": _show_parser("OPEN TABLES"),
    367            "PLUGINS": _show_parser("PLUGINS"),
    368            "PROCEDURE CODE": _show_parser("PROCEDURE CODE", target=True),
    369            "PROCEDURE STATUS": _show_parser("PROCEDURE STATUS"),
    370            "PRIVILEGES": _show_parser("PRIVILEGES"),
    371            "FULL PROCESSLIST": _show_parser("PROCESSLIST", full=True),
    372            "PROCESSLIST": _show_parser("PROCESSLIST"),
    373            "PROFILE": _show_parser("PROFILE"),
    374            "PROFILES": _show_parser("PROFILES"),
    375            "RELAYLOG EVENTS": _show_parser("RELAYLOG EVENTS"),
    376            "REPLICAS": _show_parser("REPLICAS"),
    377            "SLAVE HOSTS": _show_parser("REPLICAS"),
    378            "REPLICA STATUS": _show_parser("REPLICA STATUS"),
    379            "SLAVE STATUS": _show_parser("REPLICA STATUS"),
    380            "GLOBAL STATUS": _show_parser("STATUS", global_=True),
    381            "SESSION STATUS": _show_parser("STATUS"),
    382            "STATUS": _show_parser("STATUS"),
    383            "TABLE STATUS": _show_parser("TABLE STATUS"),
    384            "FULL TABLES": _show_parser("TABLES", full=True),
    385            "TABLES": _show_parser("TABLES"),
    386            "TRIGGERS": _show_parser("TRIGGERS"),
    387            "GLOBAL VARIABLES": _show_parser("VARIABLES", global_=True),
    388            "SESSION VARIABLES": _show_parser("VARIABLES"),
    389            "VARIABLES": _show_parser("VARIABLES"),
    390            "WARNINGS": _show_parser("WARNINGS"),
    391        }
    392
    393        PROPERTY_PARSERS = {
    394            **parser.Parser.PROPERTY_PARSERS,
    395            "LOCK": lambda self: self._parse_property_assignment(exp.LockProperty),
    396        }
    397
    398        SET_PARSERS = {
    399            **parser.Parser.SET_PARSERS,
    400            "PERSIST": lambda self: self._parse_set_item_assignment("PERSIST"),
    401            "PERSIST_ONLY": lambda self: self._parse_set_item_assignment("PERSIST_ONLY"),
    402            "CHARACTER SET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    403            "CHARSET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    404            "NAMES": lambda self: self._parse_set_item_names(),
    405        }
    406
    407        CONSTRAINT_PARSERS = {
    408            **parser.Parser.CONSTRAINT_PARSERS,
    409            "FULLTEXT": lambda self: self._parse_index_constraint(kind="FULLTEXT"),
    410            "INDEX": lambda self: self._parse_index_constraint(),
    411            "KEY": lambda self: self._parse_index_constraint(),
    412            "SPATIAL": lambda self: self._parse_index_constraint(kind="SPATIAL"),
    413        }
    414
    415        SCHEMA_UNNAMED_CONSTRAINTS = {
    416            *parser.Parser.SCHEMA_UNNAMED_CONSTRAINTS,
    417            "FULLTEXT",
    418            "INDEX",
    419            "KEY",
    420            "SPATIAL",
    421        }
    422
    423        PROFILE_TYPES: parser.OPTIONS_TYPE = {
    424            **dict.fromkeys(("ALL", "CPU", "IPC", "MEMORY", "SOURCE", "SWAPS"), tuple()),
    425            "BLOCK": ("IO",),
    426            "CONTEXT": ("SWITCHES",),
    427            "PAGE": ("FAULTS",),
    428        }
    429
    430        TYPE_TOKENS = {
    431            *parser.Parser.TYPE_TOKENS,
    432            TokenType.SET,
    433        }
    434
    435        ENUM_TYPE_TOKENS = {
    436            *parser.Parser.ENUM_TYPE_TOKENS,
    437            TokenType.SET,
    438        }
    439
    440        LOG_DEFAULTS_TO_LN = True
    441        STRING_ALIASES = True
    442        VALUES_FOLLOWED_BY_PAREN = False
    443
    444        def _parse_primary_key_part(self) -> t.Optional[exp.Expression]:
    445            this = self._parse_id_var()
    446            if not self._match(TokenType.L_PAREN):
    447                return this
    448
    449            expression = self._parse_number()
    450            self._match_r_paren()
    451            return self.expression(exp.ColumnPrefix, this=this, expression=expression)
    452
    453        def _parse_index_constraint(
    454            self, kind: t.Optional[str] = None
    455        ) -> exp.IndexColumnConstraint:
    456            if kind:
    457                self._match_texts(("INDEX", "KEY"))
    458
    459            this = self._parse_id_var(any_token=False)
    460            index_type = self._match(TokenType.USING) and self._advance_any() and self._prev.text
    461            schema = self._parse_schema()
    462
    463            options = []
    464            while True:
    465                if self._match_text_seq("KEY_BLOCK_SIZE"):
    466                    self._match(TokenType.EQ)
    467                    opt = exp.IndexConstraintOption(key_block_size=self._parse_number())
    468                elif self._match(TokenType.USING):
    469                    opt = exp.IndexConstraintOption(using=self._advance_any() and self._prev.text)
    470                elif self._match_text_seq("WITH", "PARSER"):
    471                    opt = exp.IndexConstraintOption(parser=self._parse_var(any_token=True))
    472                elif self._match(TokenType.COMMENT):
    473                    opt = exp.IndexConstraintOption(comment=self._parse_string())
    474                elif self._match_text_seq("VISIBLE"):
    475                    opt = exp.IndexConstraintOption(visible=True)
    476                elif self._match_text_seq("INVISIBLE"):
    477                    opt = exp.IndexConstraintOption(visible=False)
    478                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    479                    self._match(TokenType.EQ)
    480                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    481                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    482                    self._match(TokenType.EQ)
    483                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    484                elif self._match_text_seq("SECONDARY_ENGINE_ATTRIBUTE"):
    485                    self._match(TokenType.EQ)
    486                    opt = exp.IndexConstraintOption(secondary_engine_attr=self._parse_string())
    487                else:
    488                    opt = None
    489
    490                if not opt:
    491                    break
    492
    493                options.append(opt)
    494
    495            return self.expression(
    496                exp.IndexColumnConstraint,
    497                this=this,
    498                schema=schema,
    499                kind=kind,
    500                index_type=index_type,
    501                options=options,
    502            )
    503
    504        def _parse_show_mysql(
    505            self,
    506            this: str,
    507            target: bool | str = False,
    508            full: t.Optional[bool] = None,
    509            global_: t.Optional[bool] = None,
    510        ) -> exp.Show:
    511            if target:
    512                if isinstance(target, str):
    513                    self._match_text_seq(target)
    514                target_id = self._parse_id_var()
    515            else:
    516                target_id = None
    517
    518            log = self._parse_string() if self._match_text_seq("IN") else None
    519
    520            if this in ("BINLOG EVENTS", "RELAYLOG EVENTS"):
    521                position = self._parse_number() if self._match_text_seq("FROM") else None
    522                db = None
    523            else:
    524                position = None
    525                db = None
    526
    527                if self._match(TokenType.FROM):
    528                    db = self._parse_id_var()
    529                elif self._match(TokenType.DOT):
    530                    db = target_id
    531                    target_id = self._parse_id_var()
    532
    533            channel = self._parse_id_var() if self._match_text_seq("FOR", "CHANNEL") else None
    534
    535            like = self._parse_string() if self._match_text_seq("LIKE") else None
    536            where = self._parse_where()
    537
    538            if this == "PROFILE":
    539                types = self._parse_csv(lambda: self._parse_var_from_options(self.PROFILE_TYPES))
    540                query = self._parse_number() if self._match_text_seq("FOR", "QUERY") else None
    541                offset = self._parse_number() if self._match_text_seq("OFFSET") else None
    542                limit = self._parse_number() if self._match_text_seq("LIMIT") else None
    543            else:
    544                types, query = None, None
    545                offset, limit = self._parse_oldstyle_limit()
    546
    547            mutex = True if self._match_text_seq("MUTEX") else None
    548            mutex = False if self._match_text_seq("STATUS") else mutex
    549
    550            return self.expression(
    551                exp.Show,
    552                this=this,
    553                target=target_id,
    554                full=full,
    555                log=log,
    556                position=position,
    557                db=db,
    558                channel=channel,
    559                like=like,
    560                where=where,
    561                types=types,
    562                query=query,
    563                offset=offset,
    564                limit=limit,
    565                mutex=mutex,
    566                **{"global": global_},  # type: ignore
    567            )
    568
    569        def _parse_oldstyle_limit(
    570            self,
    571        ) -> t.Tuple[t.Optional[exp.Expression], t.Optional[exp.Expression]]:
    572            limit = None
    573            offset = None
    574            if self._match_text_seq("LIMIT"):
    575                parts = self._parse_csv(self._parse_number)
    576                if len(parts) == 1:
    577                    limit = parts[0]
    578                elif len(parts) == 2:
    579                    limit = parts[1]
    580                    offset = parts[0]
    581
    582            return offset, limit
    583
    584        def _parse_set_item_charset(self, kind: str) -> exp.Expression:
    585            this = self._parse_string() or self._parse_id_var()
    586            return self.expression(exp.SetItem, this=this, kind=kind)
    587
    588        def _parse_set_item_names(self) -> exp.Expression:
    589            charset = self._parse_string() or self._parse_id_var()
    590            if self._match_text_seq("COLLATE"):
    591                collate = self._parse_string() or self._parse_id_var()
    592            else:
    593                collate = None
    594
    595            return self.expression(exp.SetItem, this=charset, collate=collate, kind="NAMES")
    596
    597        def _parse_type(self, parse_interval: bool = True) -> t.Optional[exp.Expression]:
    598            # mysql binary is special and can work anywhere, even in order by operations
    599            # it operates like a no paren func
    600            if self._match(TokenType.BINARY, advance=False):
    601                data_type = self._parse_types(check_func=True, allow_identifiers=False)
    602
    603                if isinstance(data_type, exp.DataType):
    604                    return self.expression(exp.Cast, this=self._parse_column(), to=data_type)
    605
    606            return super()._parse_type(parse_interval=parse_interval)
    607
    608        def _parse_chr(self) -> t.Optional[exp.Expression]:
    609            expressions = self._parse_csv(self._parse_conjunction)
    610            kwargs: t.Dict[str, t.Any] = {"this": seq_get(expressions, 0)}
    611
    612            if len(expressions) > 1:
    613                kwargs["expressions"] = expressions[1:]
    614
    615            if self._match(TokenType.USING):
    616                kwargs["charset"] = self._parse_var()
    617
    618            return self.expression(exp.Chr, **kwargs)
    619
    620    class Generator(generator.Generator):
    621        LOCKING_READS_SUPPORTED = True
    622        NULL_ORDERING_SUPPORTED = None
    623        JOIN_HINTS = False
    624        TABLE_HINTS = True
    625        DUPLICATE_KEY_UPDATE_WITH_SET = False
    626        QUERY_HINT_SEP = " "
    627        VALUES_AS_TABLE = False
    628        NVL2_SUPPORTED = False
    629        LAST_DAY_SUPPORTS_DATE_PART = False
    630        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    631        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
    632        JSON_KEY_VALUE_PAIR_SEP = ","
    633        SUPPORTS_TO_NUMBER = False
    634
    635        TRANSFORMS = {
    636            **generator.Generator.TRANSFORMS,
    637            exp.CurrentDate: no_paren_current_date_sql,
    638            exp.DateDiff: _remove_ts_or_ds_to_date(
    639                lambda self, e: self.func("DATEDIFF", e.this, e.expression), ("this", "expression")
    640            ),
    641            exp.DateAdd: _remove_ts_or_ds_to_date(_date_add_sql("ADD")),
    642            exp.DateStrToDate: datestrtodate_sql,
    643            exp.DateSub: _remove_ts_or_ds_to_date(_date_add_sql("SUB")),
    644            exp.DateTrunc: _date_trunc_sql,
    645            exp.Day: _remove_ts_or_ds_to_date(),
    646            exp.DayOfMonth: _remove_ts_or_ds_to_date(rename_func("DAYOFMONTH")),
    647            exp.DayOfWeek: _remove_ts_or_ds_to_date(rename_func("DAYOFWEEK")),
    648            exp.DayOfYear: _remove_ts_or_ds_to_date(rename_func("DAYOFYEAR")),
    649            exp.GroupConcat: lambda self,
    650            e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
    651            exp.ILike: no_ilike_sql,
    652            exp.JSONExtractScalar: arrow_json_extract_sql,
    653            exp.Max: max_or_greatest,
    654            exp.Min: min_or_least,
    655            exp.Month: _remove_ts_or_ds_to_date(),
    656            exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
    657            exp.NullSafeNEQ: lambda self, e: f"NOT {self.binary(e, '<=>')}",
    658            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
    659            exp.Pivot: no_pivot_sql,
    660            exp.Select: transforms.preprocess(
    661                [
    662                    transforms.eliminate_distinct_on,
    663                    transforms.eliminate_semi_and_anti_joins,
    664                    transforms.eliminate_qualify,
    665                    transforms.eliminate_full_outer_join,
    666                ]
    667            ),
    668            exp.StrPosition: strposition_to_locate_sql,
    669            exp.StrToDate: _str_to_date_sql,
    670            exp.StrToTime: _str_to_date_sql,
    671            exp.Stuff: rename_func("INSERT"),
    672            exp.TableSample: no_tablesample_sql,
    673            exp.TimeFromParts: rename_func("MAKETIME"),
    674            exp.TimestampAdd: date_add_interval_sql("DATE", "ADD"),
    675            exp.TimestampDiff: lambda self, e: self.func(
    676                "TIMESTAMPDIFF", e.text("unit"), e.expression, e.this
    677            ),
    678            exp.TimestampSub: date_add_interval_sql("DATE", "SUB"),
    679            exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
    680            exp.TimeStrToTime: lambda self, e: self.sql(exp.cast(e.this, "datetime", copy=True)),
    681            exp.TimeToStr: _remove_ts_or_ds_to_date(
    682                lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e))
    683            ),
    684            exp.Trim: _trim_sql,
    685            exp.TryCast: no_trycast_sql,
    686            exp.TsOrDsAdd: _date_add_sql("ADD"),
    687            exp.TsOrDsDiff: lambda self, e: self.func("DATEDIFF", e.this, e.expression),
    688            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    689            exp.Week: _remove_ts_or_ds_to_date(),
    690            exp.WeekOfYear: _remove_ts_or_ds_to_date(rename_func("WEEKOFYEAR")),
    691            exp.Year: _remove_ts_or_ds_to_date(),
    692        }
    693
    694        UNSIGNED_TYPE_MAPPING = {
    695            exp.DataType.Type.UBIGINT: "BIGINT",
    696            exp.DataType.Type.UINT: "INT",
    697            exp.DataType.Type.UMEDIUMINT: "MEDIUMINT",
    698            exp.DataType.Type.USMALLINT: "SMALLINT",
    699            exp.DataType.Type.UTINYINT: "TINYINT",
    700            exp.DataType.Type.UDECIMAL: "DECIMAL",
    701        }
    702
    703        TIMESTAMP_TYPE_MAPPING = {
    704            exp.DataType.Type.TIMESTAMP: "DATETIME",
    705            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    706            exp.DataType.Type.TIMESTAMPLTZ: "TIMESTAMP",
    707        }
    708
    709        TYPE_MAPPING = {
    710            **generator.Generator.TYPE_MAPPING,
    711            **UNSIGNED_TYPE_MAPPING,
    712            **TIMESTAMP_TYPE_MAPPING,
    713        }
    714
    715        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMTEXT)
    716        TYPE_MAPPING.pop(exp.DataType.Type.LONGTEXT)
    717        TYPE_MAPPING.pop(exp.DataType.Type.TINYTEXT)
    718        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMBLOB)
    719        TYPE_MAPPING.pop(exp.DataType.Type.LONGBLOB)
    720        TYPE_MAPPING.pop(exp.DataType.Type.TINYBLOB)
    721
    722        PROPERTIES_LOCATION = {
    723            **generator.Generator.PROPERTIES_LOCATION,
    724            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    725            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    726        }
    727
    728        LIMIT_FETCH = "LIMIT"
    729
    730        LIMIT_ONLY_LITERALS = True
    731
    732        # MySQL doesn't support many datatypes in cast.
    733        # https://dev.mysql.com/doc/refman/8.0/en/cast-functions.html#function_cast
    734        CAST_MAPPING = {
    735            exp.DataType.Type.BIGINT: "SIGNED",
    736            exp.DataType.Type.BOOLEAN: "SIGNED",
    737            exp.DataType.Type.INT: "SIGNED",
    738            exp.DataType.Type.TEXT: "CHAR",
    739            exp.DataType.Type.UBIGINT: "UNSIGNED",
    740            exp.DataType.Type.VARCHAR: "CHAR",
    741        }
    742
    743        TIMESTAMP_FUNC_TYPES = {
    744            exp.DataType.Type.TIMESTAMPTZ,
    745            exp.DataType.Type.TIMESTAMPLTZ,
    746        }
    747
    748        def datatype_sql(self, expression: exp.DataType) -> str:
    749            # https://dev.mysql.com/doc/refman/8.0/en/numeric-type-syntax.html
    750            result = super().datatype_sql(expression)
    751            if expression.this in self.UNSIGNED_TYPE_MAPPING:
    752                result = f"{result} UNSIGNED"
    753            return result
    754
    755        def xor_sql(self, expression: exp.Xor) -> str:
    756            if expression.expressions:
    757                return self.expressions(expression, sep=" XOR ")
    758            return super().xor_sql(expression)
    759
    760        def jsonarraycontains_sql(self, expression: exp.JSONArrayContains) -> str:
    761            return f"{self.sql(expression, 'this')} MEMBER OF({self.sql(expression, 'expression')})"
    762
    763        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    764            if expression.to.this in self.TIMESTAMP_FUNC_TYPES:
    765                return self.func("TIMESTAMP", expression.this)
    766
    767            to = self.CAST_MAPPING.get(expression.to.this)
    768
    769            if to:
    770                expression.to.set("this", to)
    771            return super().cast_sql(expression)
    772
    773        def show_sql(self, expression: exp.Show) -> str:
    774            this = f" {expression.name}"
    775            full = " FULL" if expression.args.get("full") else ""
    776            global_ = " GLOBAL" if expression.args.get("global") else ""
    777
    778            target = self.sql(expression, "target")
    779            target = f" {target}" if target else ""
    780            if expression.name in ("COLUMNS", "INDEX"):
    781                target = f" FROM{target}"
    782            elif expression.name == "GRANTS":
    783                target = f" FOR{target}"
    784
    785            db = self._prefixed_sql("FROM", expression, "db")
    786
    787            like = self._prefixed_sql("LIKE", expression, "like")
    788            where = self.sql(expression, "where")
    789
    790            types = self.expressions(expression, key="types")
    791            types = f" {types}" if types else types
    792            query = self._prefixed_sql("FOR QUERY", expression, "query")
    793
    794            if expression.name == "PROFILE":
    795                offset = self._prefixed_sql("OFFSET", expression, "offset")
    796                limit = self._prefixed_sql("LIMIT", expression, "limit")
    797            else:
    798                offset = ""
    799                limit = self._oldstyle_limit_sql(expression)
    800
    801            log = self._prefixed_sql("IN", expression, "log")
    802            position = self._prefixed_sql("FROM", expression, "position")
    803
    804            channel = self._prefixed_sql("FOR CHANNEL", expression, "channel")
    805
    806            if expression.name == "ENGINE":
    807                mutex_or_status = " MUTEX" if expression.args.get("mutex") else " STATUS"
    808            else:
    809                mutex_or_status = ""
    810
    811            return f"SHOW{full}{global_}{this}{target}{types}{db}{query}{log}{position}{channel}{mutex_or_status}{like}{where}{offset}{limit}"
    812
    813        def altercolumn_sql(self, expression: exp.AlterColumn) -> str:
    814            dtype = self.sql(expression, "dtype")
    815            if not dtype:
    816                return super().altercolumn_sql(expression)
    817
    818            this = self.sql(expression, "this")
    819            return f"MODIFY COLUMN {this} {dtype}"
    820
    821        def _prefixed_sql(self, prefix: str, expression: exp.Expression, arg: str) -> str:
    822            sql = self.sql(expression, arg)
    823            return f" {prefix} {sql}" if sql else ""
    824
    825        def _oldstyle_limit_sql(self, expression: exp.Show) -> str:
    826            limit = self.sql(expression, "limit")
    827            offset = self.sql(expression, "offset")
    828            if limit:
    829                limit_offset = f"{offset}, {limit}" if offset else limit
    830                return f" LIMIT {limit_offset}"
    831            return ""
    832
    833        def chr_sql(self, expression: exp.Chr) -> str:
    834            this = self.expressions(sqls=[expression.this] + expression.expressions)
    835            charset = expression.args.get("charset")
    836            using = f" USING {self.sql(charset)}" if charset else ""
    837            return f"CHAR({this}{using})"

TIME_SPECIFIERS = {'f', 's', 'k', 'H', 'h', 'I', 'l', 'i', 'S', 'T', 'r', 'p'}
class MySQL(sqlglot.dialects.dialect.Dialect): View Source

    146class MySQL(Dialect):
    147    # https://dev.mysql.com/doc/refman/8.0/en/identifiers.html
    148    IDENTIFIERS_CAN_START_WITH_DIGIT = True
    149
    150    # We default to treating all identifiers as case-sensitive, since it matches MySQL's
    151    # behavior on Linux systems. For MacOS and Windows systems, one can override this
    152    # setting by specifying `dialect="mysql, normalization_strategy = lowercase"`.
    153    #
    154    # See also https://dev.mysql.com/doc/refman/8.2/en/identifier-case-sensitivity.html
    155    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_SENSITIVE
    156
    157    TIME_FORMAT = "'%Y-%m-%d %T'"
    158    DPIPE_IS_STRING_CONCAT = False
    159    SUPPORTS_USER_DEFINED_TYPES = False
    160    SUPPORTS_SEMI_ANTI_JOIN = False
    161    SAFE_DIVISION = True
    162
    163    # https://prestodb.io/docs/current/functions/datetime.html#mysql-date-functions
    164    TIME_MAPPING = {
    165        "%M": "%B",
    166        "%c": "%-m",
    167        "%e": "%-d",
    168        "%h": "%I",
    169        "%i": "%M",
    170        "%s": "%S",
    171        "%u": "%W",
    172        "%k": "%-H",
    173        "%l": "%-I",
    174        "%T": "%H:%M:%S",
    175        "%W": "%a",
    176    }
    177
    178    class Tokenizer(tokens.Tokenizer):
    179        QUOTES = ["'", '"']
    180        COMMENTS = ["--", "#", ("/*", "*/")]
    181        IDENTIFIERS = ["`"]
    182        STRING_ESCAPES = ["'", '"', "\\"]
    183        BIT_STRINGS = [("b'", "'"), ("B'", "'"), ("0b", "")]
    184        HEX_STRINGS = [("x'", "'"), ("X'", "'"), ("0x", "")]
    185
    186        KEYWORDS = {
    187            **tokens.Tokenizer.KEYWORDS,
    188            "CHARSET": TokenType.CHARACTER_SET,
    189            "FORCE": TokenType.FORCE,
    190            "IGNORE": TokenType.IGNORE,
    191            "LOCK TABLES": TokenType.COMMAND,
    192            "LONGBLOB": TokenType.LONGBLOB,
    193            "LONGTEXT": TokenType.LONGTEXT,
    194            "MEDIUMBLOB": TokenType.MEDIUMBLOB,
    195            "TINYBLOB": TokenType.TINYBLOB,
    196            "TINYTEXT": TokenType.TINYTEXT,
    197            "MEDIUMTEXT": TokenType.MEDIUMTEXT,
    198            "MEDIUMINT": TokenType.MEDIUMINT,
    199            "MEMBER OF": TokenType.MEMBER_OF,
    200            "SEPARATOR": TokenType.SEPARATOR,
    201            "START": TokenType.BEGIN,
    202            "SIGNED": TokenType.BIGINT,
    203            "SIGNED INTEGER": TokenType.BIGINT,
    204            "UNLOCK TABLES": TokenType.COMMAND,
    205            "UNSIGNED": TokenType.UBIGINT,
    206            "UNSIGNED INTEGER": TokenType.UBIGINT,
    207            "YEAR": TokenType.YEAR,
    208            "_ARMSCII8": TokenType.INTRODUCER,
    209            "_ASCII": TokenType.INTRODUCER,
    210            "_BIG5": TokenType.INTRODUCER,
    211            "_BINARY": TokenType.INTRODUCER,
    212            "_CP1250": TokenType.INTRODUCER,
    213            "_CP1251": TokenType.INTRODUCER,
    214            "_CP1256": TokenType.INTRODUCER,
    215            "_CP1257": TokenType.INTRODUCER,
    216            "_CP850": TokenType.INTRODUCER,
    217            "_CP852": TokenType.INTRODUCER,
    218            "_CP866": TokenType.INTRODUCER,
    219            "_CP932": TokenType.INTRODUCER,
    220            "_DEC8": TokenType.INTRODUCER,
    221            "_EUCJPMS": TokenType.INTRODUCER,
    222            "_EUCKR": TokenType.INTRODUCER,
    223            "_GB18030": TokenType.INTRODUCER,
    224            "_GB2312": TokenType.INTRODUCER,
    225            "_GBK": TokenType.INTRODUCER,
    226            "_GEOSTD8": TokenType.INTRODUCER,
    227            "_GREEK": TokenType.INTRODUCER,
    228            "_HEBREW": TokenType.INTRODUCER,
    229            "_HP8": TokenType.INTRODUCER,
    230            "_KEYBCS2": TokenType.INTRODUCER,
    231            "_KOI8R": TokenType.INTRODUCER,
    232            "_KOI8U": TokenType.INTRODUCER,
    233            "_LATIN1": TokenType.INTRODUCER,
    234            "_LATIN2": TokenType.INTRODUCER,
    235            "_LATIN5": TokenType.INTRODUCER,
    236            "_LATIN7": TokenType.INTRODUCER,
    237            "_MACCE": TokenType.INTRODUCER,
    238            "_MACROMAN": TokenType.INTRODUCER,
    239            "_SJIS": TokenType.INTRODUCER,
    240            "_SWE7": TokenType.INTRODUCER,
    241            "_TIS620": TokenType.INTRODUCER,
    242            "_UCS2": TokenType.INTRODUCER,
    243            "_UJIS": TokenType.INTRODUCER,
    244            # https://dev.mysql.com/doc/refman/8.0/en/string-literals.html
    245            "_UTF8": TokenType.INTRODUCER,
    246            "_UTF16": TokenType.INTRODUCER,
    247            "_UTF16LE": TokenType.INTRODUCER,
    248            "_UTF32": TokenType.INTRODUCER,
    249            "_UTF8MB3": TokenType.INTRODUCER,
    250            "_UTF8MB4": TokenType.INTRODUCER,
    251            "@@": TokenType.SESSION_PARAMETER,
    252        }
    253
    254        COMMANDS = tokens.Tokenizer.COMMANDS - {TokenType.SHOW}
    255
    256    class Parser(parser.Parser):
    257        FUNC_TOKENS = {
    258            *parser.Parser.FUNC_TOKENS,
    259            TokenType.DATABASE,
    260            TokenType.SCHEMA,
    261            TokenType.VALUES,
    262        }
    263
    264        CONJUNCTION = {
    265            **parser.Parser.CONJUNCTION,
    266            TokenType.DAMP: exp.And,
    267            TokenType.XOR: exp.Xor,
    268            TokenType.DPIPE: exp.Or,
    269        }
    270
    271        TABLE_ALIAS_TOKENS = (
    272            parser.Parser.TABLE_ALIAS_TOKENS - parser.Parser.TABLE_INDEX_HINT_TOKENS
    273        )
    274
    275        RANGE_PARSERS = {
    276            **parser.Parser.RANGE_PARSERS,
    277            TokenType.MEMBER_OF: lambda self, this: self.expression(
    278                exp.JSONArrayContains,
    279                this=this,
    280                expression=self._parse_wrapped(self._parse_expression),
    281            ),
    282        }
    283
    284        FUNCTIONS = {
    285            **parser.Parser.FUNCTIONS,
    286            "DATE": lambda args: exp.TsOrDsToDate(this=seq_get(args, 0)),
    287            "DATE_ADD": build_date_delta_with_interval(exp.DateAdd),
    288            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "mysql"),
    289            "DATE_SUB": build_date_delta_with_interval(exp.DateSub),
    290            "DAY": lambda args: exp.Day(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    291            "DAYOFMONTH": lambda args: exp.DayOfMonth(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    292            "DAYOFWEEK": lambda args: exp.DayOfWeek(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    293            "DAYOFYEAR": lambda args: exp.DayOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    294            "INSTR": lambda args: exp.StrPosition(substr=seq_get(args, 1), this=seq_get(args, 0)),
    295            "ISNULL": isnull_to_is_null,
    296            "LOCATE": locate_to_strposition,
    297            "MAKETIME": exp.TimeFromParts.from_arg_list,
    298            "MONTH": lambda args: exp.Month(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    299            "MONTHNAME": lambda args: exp.TimeToStr(
    300                this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    301                format=exp.Literal.string("%B"),
    302            ),
    303            "STR_TO_DATE": _str_to_date,
    304            "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
    305            "TO_DAYS": lambda args: exp.paren(
    306                exp.DateDiff(
    307                    this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    308                    expression=exp.TsOrDsToDate(this=exp.Literal.string("0000-01-01")),
    309                    unit=exp.var("DAY"),
    310                )
    311                + 1
    312            ),
    313            "WEEK": lambda args: exp.Week(
    314                this=exp.TsOrDsToDate(this=seq_get(args, 0)), mode=seq_get(args, 1)
    315            ),
    316            "WEEKOFYEAR": lambda args: exp.WeekOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    317            "YEAR": lambda args: exp.Year(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    318        }
    319
    320        FUNCTION_PARSERS = {
    321            **parser.Parser.FUNCTION_PARSERS,
    322            "CHAR": lambda self: self._parse_chr(),
    323            "GROUP_CONCAT": lambda self: self.expression(
    324                exp.GroupConcat,
    325                this=self._parse_lambda(),
    326                separator=self._match(TokenType.SEPARATOR) and self._parse_field(),
    327            ),
    328            # https://dev.mysql.com/doc/refman/5.7/en/miscellaneous-functions.html#function_values
    329            "VALUES": lambda self: self.expression(
    330                exp.Anonymous, this="VALUES", expressions=[self._parse_id_var()]
    331            ),
    332        }
    333
    334        STATEMENT_PARSERS = {
    335            **parser.Parser.STATEMENT_PARSERS,
    336            TokenType.SHOW: lambda self: self._parse_show(),
    337        }
    338
    339        SHOW_PARSERS = {
    340            "BINARY LOGS": _show_parser("BINARY LOGS"),
    341            "MASTER LOGS": _show_parser("BINARY LOGS"),
    342            "BINLOG EVENTS": _show_parser("BINLOG EVENTS"),
    343            "CHARACTER SET": _show_parser("CHARACTER SET"),
    344            "CHARSET": _show_parser("CHARACTER SET"),
    345            "COLLATION": _show_parser("COLLATION"),
    346            "FULL COLUMNS": _show_parser("COLUMNS", target="FROM", full=True),
    347            "COLUMNS": _show_parser("COLUMNS", target="FROM"),
    348            "CREATE DATABASE": _show_parser("CREATE DATABASE", target=True),
    349            "CREATE EVENT": _show_parser("CREATE EVENT", target=True),
    350            "CREATE FUNCTION": _show_parser("CREATE FUNCTION", target=True),
    351            "CREATE PROCEDURE": _show_parser("CREATE PROCEDURE", target=True),
    352            "CREATE TABLE": _show_parser("CREATE TABLE", target=True),
    353            "CREATE TRIGGER": _show_parser("CREATE TRIGGER", target=True),
    354            "CREATE VIEW": _show_parser("CREATE VIEW", target=True),
    355            "DATABASES": _show_parser("DATABASES"),
    356            "SCHEMAS": _show_parser("DATABASES"),
    357            "ENGINE": _show_parser("ENGINE", target=True),
    358            "STORAGE ENGINES": _show_parser("ENGINES"),
    359            "ENGINES": _show_parser("ENGINES"),
    360            "ERRORS": _show_parser("ERRORS"),
    361            "EVENTS": _show_parser("EVENTS"),
    362            "FUNCTION CODE": _show_parser("FUNCTION CODE", target=True),
    363            "FUNCTION STATUS": _show_parser("FUNCTION STATUS"),
    364            "GRANTS": _show_parser("GRANTS", target="FOR"),
    365            "INDEX": _show_parser("INDEX", target="FROM"),
    366            "MASTER STATUS": _show_parser("MASTER STATUS"),
    367            "OPEN TABLES": _show_parser("OPEN TABLES"),
    368            "PLUGINS": _show_parser("PLUGINS"),
    369            "PROCEDURE CODE": _show_parser("PROCEDURE CODE", target=True),
    370            "PROCEDURE STATUS": _show_parser("PROCEDURE STATUS"),
    371            "PRIVILEGES": _show_parser("PRIVILEGES"),
    372            "FULL PROCESSLIST": _show_parser("PROCESSLIST", full=True),
    373            "PROCESSLIST": _show_parser("PROCESSLIST"),
    374            "PROFILE": _show_parser("PROFILE"),
    375            "PROFILES": _show_parser("PROFILES"),
    376            "RELAYLOG EVENTS": _show_parser("RELAYLOG EVENTS"),
    377            "REPLICAS": _show_parser("REPLICAS"),
    378            "SLAVE HOSTS": _show_parser("REPLICAS"),
    379            "REPLICA STATUS": _show_parser("REPLICA STATUS"),
    380            "SLAVE STATUS": _show_parser("REPLICA STATUS"),
    381            "GLOBAL STATUS": _show_parser("STATUS", global_=True),
    382            "SESSION STATUS": _show_parser("STATUS"),
    383            "STATUS": _show_parser("STATUS"),
    384            "TABLE STATUS": _show_parser("TABLE STATUS"),
    385            "FULL TABLES": _show_parser("TABLES", full=True),
    386            "TABLES": _show_parser("TABLES"),
    387            "TRIGGERS": _show_parser("TRIGGERS"),
    388            "GLOBAL VARIABLES": _show_parser("VARIABLES", global_=True),
    389            "SESSION VARIABLES": _show_parser("VARIABLES"),
    390            "VARIABLES": _show_parser("VARIABLES"),
    391            "WARNINGS": _show_parser("WARNINGS"),
    392        }
    393
    394        PROPERTY_PARSERS = {
    395            **parser.Parser.PROPERTY_PARSERS,
    396            "LOCK": lambda self: self._parse_property_assignment(exp.LockProperty),
    397        }
    398
    399        SET_PARSERS = {
    400            **parser.Parser.SET_PARSERS,
    401            "PERSIST": lambda self: self._parse_set_item_assignment("PERSIST"),
    402            "PERSIST_ONLY": lambda self: self._parse_set_item_assignment("PERSIST_ONLY"),
    403            "CHARACTER SET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    404            "CHARSET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    405            "NAMES": lambda self: self._parse_set_item_names(),
    406        }
    407
    408        CONSTRAINT_PARSERS = {
    409            **parser.Parser.CONSTRAINT_PARSERS,
    410            "FULLTEXT": lambda self: self._parse_index_constraint(kind="FULLTEXT"),
    411            "INDEX": lambda self: self._parse_index_constraint(),
    412            "KEY": lambda self: self._parse_index_constraint(),
    413            "SPATIAL": lambda self: self._parse_index_constraint(kind="SPATIAL"),
    414        }
    415
    416        SCHEMA_UNNAMED_CONSTRAINTS = {
    417            *parser.Parser.SCHEMA_UNNAMED_CONSTRAINTS,
    418            "FULLTEXT",
    419            "INDEX",
    420            "KEY",
    421            "SPATIAL",
    422        }
    423
    424        PROFILE_TYPES: parser.OPTIONS_TYPE = {
    425            **dict.fromkeys(("ALL", "CPU", "IPC", "MEMORY", "SOURCE", "SWAPS"), tuple()),
    426            "BLOCK": ("IO",),
    427            "CONTEXT": ("SWITCHES",),
    428            "PAGE": ("FAULTS",),
    429        }
    430
    431        TYPE_TOKENS = {
    432            *parser.Parser.TYPE_TOKENS,
    433            TokenType.SET,
    434        }
    435
    436        ENUM_TYPE_TOKENS = {
    437            *parser.Parser.ENUM_TYPE_TOKENS,
    438            TokenType.SET,
    439        }
    440
    441        LOG_DEFAULTS_TO_LN = True
    442        STRING_ALIASES = True
    443        VALUES_FOLLOWED_BY_PAREN = False
    444
    445        def _parse_primary_key_part(self) -> t.Optional[exp.Expression]:
    446            this = self._parse_id_var()
    447            if not self._match(TokenType.L_PAREN):
    448                return this
    449
    450            expression = self._parse_number()
    451            self._match_r_paren()
    452            return self.expression(exp.ColumnPrefix, this=this, expression=expression)
    453
    454        def _parse_index_constraint(
    455            self, kind: t.Optional[str] = None
    456        ) -> exp.IndexColumnConstraint:
    457            if kind:
    458                self._match_texts(("INDEX", "KEY"))
    459
    460            this = self._parse_id_var(any_token=False)
    461            index_type = self._match(TokenType.USING) and self._advance_any() and self._prev.text
    462            schema = self._parse_schema()
    463
    464            options = []
    465            while True:
    466                if self._match_text_seq("KEY_BLOCK_SIZE"):
    467                    self._match(TokenType.EQ)
    468                    opt = exp.IndexConstraintOption(key_block_size=self._parse_number())
    469                elif self._match(TokenType.USING):
    470                    opt = exp.IndexConstraintOption(using=self._advance_any() and self._prev.text)
    471                elif self._match_text_seq("WITH", "PARSER"):
    472                    opt = exp.IndexConstraintOption(parser=self._parse_var(any_token=True))
    473                elif self._match(TokenType.COMMENT):
    474                    opt = exp.IndexConstraintOption(comment=self._parse_string())
    475                elif self._match_text_seq("VISIBLE"):
    476                    opt = exp.IndexConstraintOption(visible=True)
    477                elif self._match_text_seq("INVISIBLE"):
    478                    opt = exp.IndexConstraintOption(visible=False)
    479                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    480                    self._match(TokenType.EQ)
    481                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    482                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    483                    self._match(TokenType.EQ)
    484                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    485                elif self._match_text_seq("SECONDARY_ENGINE_ATTRIBUTE"):
    486                    self._match(TokenType.EQ)
    487                    opt = exp.IndexConstraintOption(secondary_engine_attr=self._parse_string())
    488                else:
    489                    opt = None
    490
    491                if not opt:
    492                    break
    493
    494                options.append(opt)
    495
    496            return self.expression(
    497                exp.IndexColumnConstraint,
    498                this=this,
    499                schema=schema,
    500                kind=kind,
    501                index_type=index_type,
    502                options=options,
    503            )
    504
    505        def _parse_show_mysql(
    506            self,
    507            this: str,
    508            target: bool | str = False,
    509            full: t.Optional[bool] = None,
    510            global_: t.Optional[bool] = None,
    511        ) -> exp.Show:
    512            if target:
    513                if isinstance(target, str):
    514                    self._match_text_seq(target)
    515                target_id = self._parse_id_var()
    516            else:
    517                target_id = None
    518
    519            log = self._parse_string() if self._match_text_seq("IN") else None
    520
    521            if this in ("BINLOG EVENTS", "RELAYLOG EVENTS"):
    522                position = self._parse_number() if self._match_text_seq("FROM") else None
    523                db = None
    524            else:
    525                position = None
    526                db = None
    527
    528                if self._match(TokenType.FROM):
    529                    db = self._parse_id_var()
    530                elif self._match(TokenType.DOT):
    531                    db = target_id
    532                    target_id = self._parse_id_var()
    533
    534            channel = self._parse_id_var() if self._match_text_seq("FOR", "CHANNEL") else None
    535
    536            like = self._parse_string() if self._match_text_seq("LIKE") else None
    537            where = self._parse_where()
    538
    539            if this == "PROFILE":
    540                types = self._parse_csv(lambda: self._parse_var_from_options(self.PROFILE_TYPES))
    541                query = self._parse_number() if self._match_text_seq("FOR", "QUERY") else None
    542                offset = self._parse_number() if self._match_text_seq("OFFSET") else None
    543                limit = self._parse_number() if self._match_text_seq("LIMIT") else None
    544            else:
    545                types, query = None, None
    546                offset, limit = self._parse_oldstyle_limit()
    547
    548            mutex = True if self._match_text_seq("MUTEX") else None
    549            mutex = False if self._match_text_seq("STATUS") else mutex
    550
    551            return self.expression(
    552                exp.Show,
    553                this=this,
    554                target=target_id,
    555                full=full,
    556                log=log,
    557                position=position,
    558                db=db,
    559                channel=channel,
    560                like=like,
    561                where=where,
    562                types=types,
    563                query=query,
    564                offset=offset,
    565                limit=limit,
    566                mutex=mutex,
    567                **{"global": global_},  # type: ignore
    568            )
    569
    570        def _parse_oldstyle_limit(
    571            self,
    572        ) -> t.Tuple[t.Optional[exp.Expression], t.Optional[exp.Expression]]:
    573            limit = None
    574            offset = None
    575            if self._match_text_seq("LIMIT"):
    576                parts = self._parse_csv(self._parse_number)
    577                if len(parts) == 1:
    578                    limit = parts[0]
    579                elif len(parts) == 2:
    580                    limit = parts[1]
    581                    offset = parts[0]
    582
    583            return offset, limit
    584
    585        def _parse_set_item_charset(self, kind: str) -> exp.Expression:
    586            this = self._parse_string() or self._parse_id_var()
    587            return self.expression(exp.SetItem, this=this, kind=kind)
    588
    589        def _parse_set_item_names(self) -> exp.Expression:
    590            charset = self._parse_string() or self._parse_id_var()
    591            if self._match_text_seq("COLLATE"):
    592                collate = self._parse_string() or self._parse_id_var()
    593            else:
    594                collate = None
    595
    596            return self.expression(exp.SetItem, this=charset, collate=collate, kind="NAMES")
    597
    598        def _parse_type(self, parse_interval: bool = True) -> t.Optional[exp.Expression]:
    599            # mysql binary is special and can work anywhere, even in order by operations
    600            # it operates like a no paren func
    601            if self._match(TokenType.BINARY, advance=False):
    602                data_type = self._parse_types(check_func=True, allow_identifiers=False)
    603
    604                if isinstance(data_type, exp.DataType):
    605                    return self.expression(exp.Cast, this=self._parse_column(), to=data_type)
    606
    607            return super()._parse_type(parse_interval=parse_interval)
    608
    609        def _parse_chr(self) -> t.Optional[exp.Expression]:
    610            expressions = self._parse_csv(self._parse_conjunction)
    611            kwargs: t.Dict[str, t.Any] = {"this": seq_get(expressions, 0)}
    612
    613            if len(expressions) > 1:
    614                kwargs["expressions"] = expressions[1:]
    615
    616            if self._match(TokenType.USING):
    617                kwargs["charset"] = self._parse_var()
    618
    619            return self.expression(exp.Chr, **kwargs)
    620
    621    class Generator(generator.Generator):
    622        LOCKING_READS_SUPPORTED = True
    623        NULL_ORDERING_SUPPORTED = None
    624        JOIN_HINTS = False
    625        TABLE_HINTS = True
    626        DUPLICATE_KEY_UPDATE_WITH_SET = False
    627        QUERY_HINT_SEP = " "
    628        VALUES_AS_TABLE = False
    629        NVL2_SUPPORTED = False
    630        LAST_DAY_SUPPORTS_DATE_PART = False
    631        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    632        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
    633        JSON_KEY_VALUE_PAIR_SEP = ","
    634        SUPPORTS_TO_NUMBER = False
    635
    636        TRANSFORMS = {
    637            **generator.Generator.TRANSFORMS,
    638            exp.CurrentDate: no_paren_current_date_sql,
    639            exp.DateDiff: _remove_ts_or_ds_to_date(
    640                lambda self, e: self.func("DATEDIFF", e.this, e.expression), ("this", "expression")
    641            ),
    642            exp.DateAdd: _remove_ts_or_ds_to_date(_date_add_sql("ADD")),
    643            exp.DateStrToDate: datestrtodate_sql,
    644            exp.DateSub: _remove_ts_or_ds_to_date(_date_add_sql("SUB")),
    645            exp.DateTrunc: _date_trunc_sql,
    646            exp.Day: _remove_ts_or_ds_to_date(),
    647            exp.DayOfMonth: _remove_ts_or_ds_to_date(rename_func("DAYOFMONTH")),
    648            exp.DayOfWeek: _remove_ts_or_ds_to_date(rename_func("DAYOFWEEK")),
    649            exp.DayOfYear: _remove_ts_or_ds_to_date(rename_func("DAYOFYEAR")),
    650            exp.GroupConcat: lambda self,
    651            e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
    652            exp.ILike: no_ilike_sql,
    653            exp.JSONExtractScalar: arrow_json_extract_sql,
    654            exp.Max: max_or_greatest,
    655            exp.Min: min_or_least,
    656            exp.Month: _remove_ts_or_ds_to_date(),
    657            exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
    658            exp.NullSafeNEQ: lambda self, e: f"NOT {self.binary(e, '<=>')}",
    659            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
    660            exp.Pivot: no_pivot_sql,
    661            exp.Select: transforms.preprocess(
    662                [
    663                    transforms.eliminate_distinct_on,
    664                    transforms.eliminate_semi_and_anti_joins,
    665                    transforms.eliminate_qualify,
    666                    transforms.eliminate_full_outer_join,
    667                ]
    668            ),
    669            exp.StrPosition: strposition_to_locate_sql,
    670            exp.StrToDate: _str_to_date_sql,
    671            exp.StrToTime: _str_to_date_sql,
    672            exp.Stuff: rename_func("INSERT"),
    673            exp.TableSample: no_tablesample_sql,
    674            exp.TimeFromParts: rename_func("MAKETIME"),
    675            exp.TimestampAdd: date_add_interval_sql("DATE", "ADD"),
    676            exp.TimestampDiff: lambda self, e: self.func(
    677                "TIMESTAMPDIFF", e.text("unit"), e.expression, e.this
    678            ),
    679            exp.TimestampSub: date_add_interval_sql("DATE", "SUB"),
    680            exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
    681            exp.TimeStrToTime: lambda self, e: self.sql(exp.cast(e.this, "datetime", copy=True)),
    682            exp.TimeToStr: _remove_ts_or_ds_to_date(
    683                lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e))
    684            ),
    685            exp.Trim: _trim_sql,
    686            exp.TryCast: no_trycast_sql,
    687            exp.TsOrDsAdd: _date_add_sql("ADD"),
    688            exp.TsOrDsDiff: lambda self, e: self.func("DATEDIFF", e.this, e.expression),
    689            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    690            exp.Week: _remove_ts_or_ds_to_date(),
    691            exp.WeekOfYear: _remove_ts_or_ds_to_date(rename_func("WEEKOFYEAR")),
    692            exp.Year: _remove_ts_or_ds_to_date(),
    693        }
    694
    695        UNSIGNED_TYPE_MAPPING = {
    696            exp.DataType.Type.UBIGINT: "BIGINT",
    697            exp.DataType.Type.UINT: "INT",
    698            exp.DataType.Type.UMEDIUMINT: "MEDIUMINT",
    699            exp.DataType.Type.USMALLINT: "SMALLINT",
    700            exp.DataType.Type.UTINYINT: "TINYINT",
    701            exp.DataType.Type.UDECIMAL: "DECIMAL",
    702        }
    703
    704        TIMESTAMP_TYPE_MAPPING = {
    705            exp.DataType.Type.TIMESTAMP: "DATETIME",
    706            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    707            exp.DataType.Type.TIMESTAMPLTZ: "TIMESTAMP",
    708        }
    709
    710        TYPE_MAPPING = {
    711            **generator.Generator.TYPE_MAPPING,
    712            **UNSIGNED_TYPE_MAPPING,
    713            **TIMESTAMP_TYPE_MAPPING,
    714        }
    715
    716        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMTEXT)
    717        TYPE_MAPPING.pop(exp.DataType.Type.LONGTEXT)
    718        TYPE_MAPPING.pop(exp.DataType.Type.TINYTEXT)
    719        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMBLOB)
    720        TYPE_MAPPING.pop(exp.DataType.Type.LONGBLOB)
    721        TYPE_MAPPING.pop(exp.DataType.Type.TINYBLOB)
    722
    723        PROPERTIES_LOCATION = {
    724            **generator.Generator.PROPERTIES_LOCATION,
    725            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    726            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    727        }
    728
    729        LIMIT_FETCH = "LIMIT"
    730
    731        LIMIT_ONLY_LITERALS = True
    732
    733        # MySQL doesn't support many datatypes in cast.
    734        # https://dev.mysql.com/doc/refman/8.0/en/cast-functions.html#function_cast
    735        CAST_MAPPING = {
    736            exp.DataType.Type.BIGINT: "SIGNED",
    737            exp.DataType.Type.BOOLEAN: "SIGNED",
    738            exp.DataType.Type.INT: "SIGNED",
    739            exp.DataType.Type.TEXT: "CHAR",
    740            exp.DataType.Type.UBIGINT: "UNSIGNED",
    741            exp.DataType.Type.VARCHAR: "CHAR",
    742        }
    743
    744        TIMESTAMP_FUNC_TYPES = {
    745            exp.DataType.Type.TIMESTAMPTZ,
    746            exp.DataType.Type.TIMESTAMPLTZ,
    747        }
    748
    749        def datatype_sql(self, expression: exp.DataType) -> str:
    750            # https://dev.mysql.com/doc/refman/8.0/en/numeric-type-syntax.html
    751            result = super().datatype_sql(expression)
    752            if expression.this in self.UNSIGNED_TYPE_MAPPING:
    753                result = f"{result} UNSIGNED"
    754            return result
    755
    756        def xor_sql(self, expression: exp.Xor) -> str:
    757            if expression.expressions:
    758                return self.expressions(expression, sep=" XOR ")
    759            return super().xor_sql(expression)
    760
    761        def jsonarraycontains_sql(self, expression: exp.JSONArrayContains) -> str:
    762            return f"{self.sql(expression, 'this')} MEMBER OF({self.sql(expression, 'expression')})"
    763
    764        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    765            if expression.to.this in self.TIMESTAMP_FUNC_TYPES:
    766                return self.func("TIMESTAMP", expression.this)
    767
    768            to = self.CAST_MAPPING.get(expression.to.this)
    769
    770            if to:
    771                expression.to.set("this", to)
    772            return super().cast_sql(expression)
    773
    774        def show_sql(self, expression: exp.Show) -> str:
    775            this = f" {expression.name}"
    776            full = " FULL" if expression.args.get("full") else ""
    777            global_ = " GLOBAL" if expression.args.get("global") else ""
    778
    779            target = self.sql(expression, "target")
    780            target = f" {target}" if target else ""
    781            if expression.name in ("COLUMNS", "INDEX"):
    782                target = f" FROM{target}"
    783            elif expression.name == "GRANTS":
    784                target = f" FOR{target}"
    785
    786            db = self._prefixed_sql("FROM", expression, "db")
    787
    788            like = self._prefixed_sql("LIKE", expression, "like")
    789            where = self.sql(expression, "where")
    790
    791            types = self.expressions(expression, key="types")
    792            types = f" {types}" if types else types
    793            query = self._prefixed_sql("FOR QUERY", expression, "query")
    794
    795            if expression.name == "PROFILE":
    796                offset = self._prefixed_sql("OFFSET", expression, "offset")
    797                limit = self._prefixed_sql("LIMIT", expression, "limit")
    798            else:
    799                offset = ""
    800                limit = self._oldstyle_limit_sql(expression)
    801
    802            log = self._prefixed_sql("IN", expression, "log")
    803            position = self._prefixed_sql("FROM", expression, "position")
    804
    805            channel = self._prefixed_sql("FOR CHANNEL", expression, "channel")
    806
    807            if expression.name == "ENGINE":
    808                mutex_or_status = " MUTEX" if expression.args.get("mutex") else " STATUS"
    809            else:
    810                mutex_or_status = ""
    811
    812            return f"SHOW{full}{global_}{this}{target}{types}{db}{query}{log}{position}{channel}{mutex_or_status}{like}{where}{offset}{limit}"
    813
    814        def altercolumn_sql(self, expression: exp.AlterColumn) -> str:
    815            dtype = self.sql(expression, "dtype")
    816            if not dtype:
    817                return super().altercolumn_sql(expression)
    818
    819            this = self.sql(expression, "this")
    820            return f"MODIFY COLUMN {this} {dtype}"
    821
    822        def _prefixed_sql(self, prefix: str, expression: exp.Expression, arg: str) -> str:
    823            sql = self.sql(expression, arg)
    824            return f" {prefix} {sql}" if sql else ""
    825
    826        def _oldstyle_limit_sql(self, expression: exp.Show) -> str:
    827            limit = self.sql(expression, "limit")
    828            offset = self.sql(expression, "offset")
    829            if limit:
    830                limit_offset = f"{offset}, {limit}" if offset else limit
    831                return f" LIMIT {limit_offset}"
    832            return ""
    833
    834        def chr_sql(self, expression: exp.Chr) -> str:
    835            this = self.expressions(sqls=[expression.this] + expression.expressions)
    836            charset = expression.args.get("charset")
    837            using = f" USING {self.sql(charset)}" if charset else ""
    838            return f"CHAR({this}{using})"

IDENTIFIERS_CAN_START_WITH_DIGIT = True

Whether an unquoted identifier can start with a digit.

NORMALIZATION_STRATEGY = <NormalizationStrategy.CASE_SENSITIVE: 'CASE_SENSITIVE'>

Specifies the strategy according to which identifiers should be normalized.

TIME_FORMAT = "'%Y-%m-%d %T'"
DPIPE_IS_STRING_CONCAT = False

Whether the DPIPE token (`||`) is a string concatenation operator.

SUPPORTS_USER_DEFINED_TYPES = False

Whether user-defined data types are supported.

SUPPORTS_SEMI_ANTI_JOIN = False

Whether `SEMI` or `ANTI` joins are supported.

SAFE_DIVISION = True

Whether division by zero throws an error (`False`) or returns NULL (`True`).

TIME_MAPPING: Dict[str, str] = {'%M': '%B', '%c': '%-m', '%e': '%-d', '%h': '%I', '%i': '%M', '%s': '%S', '%u': '%W', '%k': '%-H', '%l': '%-I', '%T': '%H:%M:%S', '%W': '%a'}

Associates this dialect's time formats with their equivalent Python `strftime` formats.

tokenizer_class = <class 'MySQL.Tokenizer'>
parser_class = <class 'MySQL.Parser'>
generator_class = <class 'MySQL.Generator'>
TIME_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
FORMAT_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%B': '%M', '%-m': '%c', '%-d': '%e', '%I': '%h', '%M': '%i', '%S': '%s', '%W': '%u', '%-H': '%k', '%-I': '%l', '%H:%M:%S': '%T', '%a': '%W'}
INVERSE_TIME_TRIE: Dict = {'%': {'B': {0: True}, '-': {'m': {0: True}, 'd': {0: True}, 'H': {0: True}, 'I': {0: True}}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'W': {0: True}, 'H': {':': {'%': {'M': {':': {'%': {'S': {0: True}}}}}}}, 'a': {0: True}}}
INVERSE_ESCAPE_SEQUENCES: Dict[str, str] = {}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '`'
IDENTIFIER_END = '`'
BIT_START: Optional[str] = "b'"
BIT_END: Optional[str] = "'"
HEX_START: Optional[str] = "x'"
HEX_END: Optional[str] = "'"
BYTE_START: Optional[str] = None
BYTE_END: Optional[str] = None
UNICODE_START: Optional[str] = None
UNICODE_END: Optional[str] = None

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  Dialect
  INDEX_OFFSET
  WEEK_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  TABLESAMPLE_SIZE_IS_PERCENT
  STRICT_STRING_CONCAT
  NORMALIZE_FUNCTIONS
  LOG_BASE_FIRST
  NULL_ORDERING
  TYPED_DIVISION
  CONCAT_COALESCE
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  ESCAPE_SEQUENCES
  PSEUDOCOLUMNS
  PREFER_CTE_ALIAS_COLUMN
  get_or_raise
  format_time
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  to_json_path
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  parser
  generator

class MySQL.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    178    class Tokenizer(tokens.Tokenizer):
    179        QUOTES = ["'", '"']
    180        COMMENTS = ["--", "#", ("/*", "*/")]
    181        IDENTIFIERS = ["`"]
    182        STRING_ESCAPES = ["'", '"', "\\"]
    183        BIT_STRINGS = [("b'", "'"), ("B'", "'"), ("0b", "")]
    184        HEX_STRINGS = [("x'", "'"), ("X'", "'"), ("0x", "")]
    185
    186        KEYWORDS = {
    187            **tokens.Tokenizer.KEYWORDS,
    188            "CHARSET": TokenType.CHARACTER_SET,
    189            "FORCE": TokenType.FORCE,
    190            "IGNORE": TokenType.IGNORE,
    191            "LOCK TABLES": TokenType.COMMAND,
    192            "LONGBLOB": TokenType.LONGBLOB,
    193            "LONGTEXT": TokenType.LONGTEXT,
    194            "MEDIUMBLOB": TokenType.MEDIUMBLOB,
    195            "TINYBLOB": TokenType.TINYBLOB,
    196            "TINYTEXT": TokenType.TINYTEXT,
    197            "MEDIUMTEXT": TokenType.MEDIUMTEXT,
    198            "MEDIUMINT": TokenType.MEDIUMINT,
    199            "MEMBER OF": TokenType.MEMBER_OF,
    200            "SEPARATOR": TokenType.SEPARATOR,
    201            "START": TokenType.BEGIN,
    202            "SIGNED": TokenType.BIGINT,
    203            "SIGNED INTEGER": TokenType.BIGINT,
    204            "UNLOCK TABLES": TokenType.COMMAND,
    205            "UNSIGNED": TokenType.UBIGINT,
    206            "UNSIGNED INTEGER": TokenType.UBIGINT,
    207            "YEAR": TokenType.YEAR,
    208            "_ARMSCII8": TokenType.INTRODUCER,
    209            "_ASCII": TokenType.INTRODUCER,
    210            "_BIG5": TokenType.INTRODUCER,
    211            "_BINARY": TokenType.INTRODUCER,
    212            "_CP1250": TokenType.INTRODUCER,
    213            "_CP1251": TokenType.INTRODUCER,
    214            "_CP1256": TokenType.INTRODUCER,
    215            "_CP1257": TokenType.INTRODUCER,
    216            "_CP850": TokenType.INTRODUCER,
    217            "_CP852": TokenType.INTRODUCER,
    218            "_CP866": TokenType.INTRODUCER,
    219            "_CP932": TokenType.INTRODUCER,
    220            "_DEC8": TokenType.INTRODUCER,
    221            "_EUCJPMS": TokenType.INTRODUCER,
    222            "_EUCKR": TokenType.INTRODUCER,
    223            "_GB18030": TokenType.INTRODUCER,
    224            "_GB2312": TokenType.INTRODUCER,
    225            "_GBK": TokenType.INTRODUCER,
    226            "_GEOSTD8": TokenType.INTRODUCER,
    227            "_GREEK": TokenType.INTRODUCER,
    228            "_HEBREW": TokenType.INTRODUCER,
    229            "_HP8": TokenType.INTRODUCER,
    230            "_KEYBCS2": TokenType.INTRODUCER,
    231            "_KOI8R": TokenType.INTRODUCER,
    232            "_KOI8U": TokenType.INTRODUCER,
    233            "_LATIN1": TokenType.INTRODUCER,
    234            "_LATIN2": TokenType.INTRODUCER,
    235            "_LATIN5": TokenType.INTRODUCER,
    236            "_LATIN7": TokenType.INTRODUCER,
    237            "_MACCE": TokenType.INTRODUCER,
    238            "_MACROMAN": TokenType.INTRODUCER,
    239            "_SJIS": TokenType.INTRODUCER,
    240            "_SWE7": TokenType.INTRODUCER,
    241            "_TIS620": TokenType.INTRODUCER,
    242            "_UCS2": TokenType.INTRODUCER,
    243            "_UJIS": TokenType.INTRODUCER,
    244            # https://dev.mysql.com/doc/refman/8.0/en/string-literals.html
    245            "_UTF8": TokenType.INTRODUCER,
    246            "_UTF16": TokenType.INTRODUCER,
    247            "_UTF16LE": TokenType.INTRODUCER,
    248            "_UTF32": TokenType.INTRODUCER,
    249            "_UTF8MB3": TokenType.INTRODUCER,
    250            "_UTF8MB4": TokenType.INTRODUCER,
    251            "@@": TokenType.SESSION_PARAMETER,
    252        }
    253
    254        COMMANDS = tokens.Tokenizer.COMMANDS - {TokenType.SHOW}

QUOTES = ["'", '"']
COMMENTS = ['--', '#', ('/*', '*/')]
IDENTIFIERS = ['`']
STRING_ESCAPES = ["'", '"', '\\']
BIT_STRINGS = [("b'", "'"), ("B'", "'"), ('0b', '')]
HEX_STRINGS = [("x'", "'"), ("X'", "'"), ('0x', '')]
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '/*+': <TokenType.HINT: 'HINT'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, ':=': <TokenType.COLON_EQ: 'COLON_EQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, '??': <TokenType.DQMARK: 'DQMARK'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.BEGIN: 'BEGIN'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONNECT BY': <TokenType.CONNECT_BY: 'CONNECT_BY'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ENUM': <TokenType.ENUM: 'ENUM'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'KILL': <TokenType.KILL: 'KILL'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'XOR': <TokenType.XOR: 'XOR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'QUALIFY': <TokenType.QUALIFY: 'QUALIFY'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.ROW: 'ROW'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'START WITH': <TokenType.START_WITH: 'START_WITH'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'TRUNCATE': <TokenType.TRUNCATE: 'TRUNCATE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNKNOWN': <TokenType.UNKNOWN: 'UNKNOWN'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'MEDIUMINT': <TokenType.MEDIUMINT: 'MEDIUMINT'>, 'INT1': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'INT16': <TokenType.SMALLINT: 'SMALLINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT128': <TokenType.INT128: 'INT128'>, 'HUGEINT': <TokenType.INT128: 'INT128'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'INT32': <TokenType.INT: 'INT'>, 'INT64': <TokenType.BIGINT: 'BIGINT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.TINYINT: 'TINYINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'BPCHAR': <TokenType.BPCHAR: 'BPCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'LONGTEXT': <TokenType.LONGTEXT: 'LONGTEXT'>, 'MEDIUMTEXT': <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, 'TINYTEXT': <TokenType.TINYTEXT: 'TINYTEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'LONGBLOB': <TokenType.LONGBLOB: 'LONGBLOB'>, 'MEDIUMBLOB': <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, 'TINYBLOB': <TokenType.TINYBLOB: 'TINYBLOB'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMETZ': <TokenType.TIMETZ: 'TIMETZ'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'SEQUENCE': <TokenType.SEQUENCE: 'SEQUENCE'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'COPY': <TokenType.COMMAND: 'COMMAND'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'FOR VERSION': <TokenType.VERSION_SNAPSHOT: 'VERSION_SNAPSHOT'>, 'FOR TIMESTAMP': <TokenType.TIMESTAMP_SNAPSHOT: 'TIMESTAMP_SNAPSHOT'>, 'CHARSET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'FORCE': <TokenType.FORCE: 'FORCE'>, 'IGNORE': <TokenType.IGNORE: 'IGNORE'>, 'LOCK TABLES': <TokenType.COMMAND: 'COMMAND'>, 'MEMBER OF': <TokenType.MEMBER_OF: 'MEMBER_OF'>, 'SEPARATOR': <TokenType.SEPARATOR: 'SEPARATOR'>, 'START': <TokenType.BEGIN: 'BEGIN'>, 'SIGNED': <TokenType.BIGINT: 'BIGINT'>, 'SIGNED INTEGER': <TokenType.BIGINT: 'BIGINT'>, 'UNLOCK TABLES': <TokenType.COMMAND: 'COMMAND'>, 'UNSIGNED': <TokenType.UBIGINT: 'UBIGINT'>, 'UNSIGNED INTEGER': <TokenType.UBIGINT: 'UBIGINT'>, 'YEAR': <TokenType.YEAR: 'YEAR'>, '_ARMSCII8': <TokenType.INTRODUCER: 'INTRODUCER'>, '_ASCII': <TokenType.INTRODUCER: 'INTRODUCER'>, '_BIG5': <TokenType.INTRODUCER: 'INTRODUCER'>, '_BINARY': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP1250': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP1251': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP1256': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP1257': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP850': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP852': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP866': <TokenType.INTRODUCER: 'INTRODUCER'>, '_CP932': <TokenType.INTRODUCER: 'INTRODUCER'>, '_DEC8': <TokenType.INTRODUCER: 'INTRODUCER'>, '_EUCJPMS': <TokenType.INTRODUCER: 'INTRODUCER'>, '_EUCKR': <TokenType.INTRODUCER: 'INTRODUCER'>, '_GB18030': <TokenType.INTRODUCER: 'INTRODUCER'>, '_GB2312': <TokenType.INTRODUCER: 'INTRODUCER'>, '_GBK': <TokenType.INTRODUCER: 'INTRODUCER'>, '_GEOSTD8': <TokenType.INTRODUCER: 'INTRODUCER'>, '_GREEK': <TokenType.INTRODUCER: 'INTRODUCER'>, '_HEBREW': <TokenType.INTRODUCER: 'INTRODUCER'>, '_HP8': <TokenType.INTRODUCER: 'INTRODUCER'>, '_KEYBCS2': <TokenType.INTRODUCER: 'INTRODUCER'>, '_KOI8R': <TokenType.INTRODUCER: 'INTRODUCER'>, '_KOI8U': <TokenType.INTRODUCER: 'INTRODUCER'>, '_LATIN1': <TokenType.INTRODUCER: 'INTRODUCER'>, '_LATIN2': <TokenType.INTRODUCER: 'INTRODUCER'>, '_LATIN5': <TokenType.INTRODUCER: 'INTRODUCER'>, '_LATIN7': <TokenType.INTRODUCER: 'INTRODUCER'>, '_MACCE': <TokenType.INTRODUCER: 'INTRODUCER'>, '_MACROMAN': <TokenType.INTRODUCER: 'INTRODUCER'>, '_SJIS': <TokenType.INTRODUCER: 'INTRODUCER'>, '_SWE7': <TokenType.INTRODUCER: 'INTRODUCER'>, '_TIS620': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UCS2': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UJIS': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF8': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF16': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF16LE': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF32': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF8MB3': <TokenType.INTRODUCER: 'INTRODUCER'>, '_UTF8MB4': <TokenType.INTRODUCER: 'INTRODUCER'>, '@@': <TokenType.SESSION_PARAMETER: 'SESSION_PARAMETER'>}
COMMANDS = {<TokenType.COMMAND: 'COMMAND'>, <TokenType.FETCH: 'FETCH'>, <TokenType.EXECUTE: 'EXECUTE'>}

##### Inherited Members

sqlglot.tokens.Tokenizer
  Tokenizer
  SINGLE_TOKENS
  BYTE_STRINGS
  RAW_STRINGS
  HEREDOC_STRINGS
  UNICODE_STRINGS
  IDENTIFIER_ESCAPES
  VAR_SINGLE_TOKENS
  HEREDOC_TAG_IS_IDENTIFIER
  HEREDOC_STRING_ALTERNATIVE
  WHITE_SPACE
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  dialect
  reset
  tokenize
  tokenize_rs
  size
  sql
  tokens

class MySQL.Parser(sqlglot.parser.Parser): View Source

    256    class Parser(parser.Parser):
    257        FUNC_TOKENS = {
    258            *parser.Parser.FUNC_TOKENS,
    259            TokenType.DATABASE,
    260            TokenType.SCHEMA,
    261            TokenType.VALUES,
    262        }
    263
    264        CONJUNCTION = {
    265            **parser.Parser.CONJUNCTION,
    266            TokenType.DAMP: exp.And,
    267            TokenType.XOR: exp.Xor,
    268            TokenType.DPIPE: exp.Or,
    269        }
    270
    271        TABLE_ALIAS_TOKENS = (
    272            parser.Parser.TABLE_ALIAS_TOKENS - parser.Parser.TABLE_INDEX_HINT_TOKENS
    273        )
    274
    275        RANGE_PARSERS = {
    276            **parser.Parser.RANGE_PARSERS,
    277            TokenType.MEMBER_OF: lambda self, this: self.expression(
    278                exp.JSONArrayContains,
    279                this=this,
    280                expression=self._parse_wrapped(self._parse_expression),
    281            ),
    282        }
    283
    284        FUNCTIONS = {
    285            **parser.Parser.FUNCTIONS,
    286            "DATE": lambda args: exp.TsOrDsToDate(this=seq_get(args, 0)),
    287            "DATE_ADD": build_date_delta_with_interval(exp.DateAdd),
    288            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "mysql"),
    289            "DATE_SUB": build_date_delta_with_interval(exp.DateSub),
    290            "DAY": lambda args: exp.Day(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    291            "DAYOFMONTH": lambda args: exp.DayOfMonth(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    292            "DAYOFWEEK": lambda args: exp.DayOfWeek(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    293            "DAYOFYEAR": lambda args: exp.DayOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    294            "INSTR": lambda args: exp.StrPosition(substr=seq_get(args, 1), this=seq_get(args, 0)),
    295            "ISNULL": isnull_to_is_null,
    296            "LOCATE": locate_to_strposition,
    297            "MAKETIME": exp.TimeFromParts.from_arg_list,
    298            "MONTH": lambda args: exp.Month(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    299            "MONTHNAME": lambda args: exp.TimeToStr(
    300                this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    301                format=exp.Literal.string("%B"),
    302            ),
    303            "STR_TO_DATE": _str_to_date,
    304            "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
    305            "TO_DAYS": lambda args: exp.paren(
    306                exp.DateDiff(
    307                    this=exp.TsOrDsToDate(this=seq_get(args, 0)),
    308                    expression=exp.TsOrDsToDate(this=exp.Literal.string("0000-01-01")),
    309                    unit=exp.var("DAY"),
    310                )
    311                + 1
    312            ),
    313            "WEEK": lambda args: exp.Week(
    314                this=exp.TsOrDsToDate(this=seq_get(args, 0)), mode=seq_get(args, 1)
    315            ),
    316            "WEEKOFYEAR": lambda args: exp.WeekOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    317            "YEAR": lambda args: exp.Year(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
    318        }
    319
    320        FUNCTION_PARSERS = {
    321            **parser.Parser.FUNCTION_PARSERS,
    322            "CHAR": lambda self: self._parse_chr(),
    323            "GROUP_CONCAT": lambda self: self.expression(
    324                exp.GroupConcat,
    325                this=self._parse_lambda(),
    326                separator=self._match(TokenType.SEPARATOR) and self._parse_field(),
    327            ),
    328            # https://dev.mysql.com/doc/refman/5.7/en/miscellaneous-functions.html#function_values
    329            "VALUES": lambda self: self.expression(
    330                exp.Anonymous, this="VALUES", expressions=[self._parse_id_var()]
    331            ),
    332        }
    333
    334        STATEMENT_PARSERS = {
    335            **parser.Parser.STATEMENT_PARSERS,
    336            TokenType.SHOW: lambda self: self._parse_show(),
    337        }
    338
    339        SHOW_PARSERS = {
    340            "BINARY LOGS": _show_parser("BINARY LOGS"),
    341            "MASTER LOGS": _show_parser("BINARY LOGS"),
    342            "BINLOG EVENTS": _show_parser("BINLOG EVENTS"),
    343            "CHARACTER SET": _show_parser("CHARACTER SET"),
    344            "CHARSET": _show_parser("CHARACTER SET"),
    345            "COLLATION": _show_parser("COLLATION"),
    346            "FULL COLUMNS": _show_parser("COLUMNS", target="FROM", full=True),
    347            "COLUMNS": _show_parser("COLUMNS", target="FROM"),
    348            "CREATE DATABASE": _show_parser("CREATE DATABASE", target=True),
    349            "CREATE EVENT": _show_parser("CREATE EVENT", target=True),
    350            "CREATE FUNCTION": _show_parser("CREATE FUNCTION", target=True),
    351            "CREATE PROCEDURE": _show_parser("CREATE PROCEDURE", target=True),
    352            "CREATE TABLE": _show_parser("CREATE TABLE", target=True),
    353            "CREATE TRIGGER": _show_parser("CREATE TRIGGER", target=True),
    354            "CREATE VIEW": _show_parser("CREATE VIEW", target=True),
    355            "DATABASES": _show_parser("DATABASES"),
    356            "SCHEMAS": _show_parser("DATABASES"),
    357            "ENGINE": _show_parser("ENGINE", target=True),
    358            "STORAGE ENGINES": _show_parser("ENGINES"),
    359            "ENGINES": _show_parser("ENGINES"),
    360            "ERRORS": _show_parser("ERRORS"),
    361            "EVENTS": _show_parser("EVENTS"),
    362            "FUNCTION CODE": _show_parser("FUNCTION CODE", target=True),
    363            "FUNCTION STATUS": _show_parser("FUNCTION STATUS"),
    364            "GRANTS": _show_parser("GRANTS", target="FOR"),
    365            "INDEX": _show_parser("INDEX", target="FROM"),
    366            "MASTER STATUS": _show_parser("MASTER STATUS"),
    367            "OPEN TABLES": _show_parser("OPEN TABLES"),
    368            "PLUGINS": _show_parser("PLUGINS"),
    369            "PROCEDURE CODE": _show_parser("PROCEDURE CODE", target=True),
    370            "PROCEDURE STATUS": _show_parser("PROCEDURE STATUS"),
    371            "PRIVILEGES": _show_parser("PRIVILEGES"),
    372            "FULL PROCESSLIST": _show_parser("PROCESSLIST", full=True),
    373            "PROCESSLIST": _show_parser("PROCESSLIST"),
    374            "PROFILE": _show_parser("PROFILE"),
    375            "PROFILES": _show_parser("PROFILES"),
    376            "RELAYLOG EVENTS": _show_parser("RELAYLOG EVENTS"),
    377            "REPLICAS": _show_parser("REPLICAS"),
    378            "SLAVE HOSTS": _show_parser("REPLICAS"),
    379            "REPLICA STATUS": _show_parser("REPLICA STATUS"),
    380            "SLAVE STATUS": _show_parser("REPLICA STATUS"),
    381            "GLOBAL STATUS": _show_parser("STATUS", global_=True),
    382            "SESSION STATUS": _show_parser("STATUS"),
    383            "STATUS": _show_parser("STATUS"),
    384            "TABLE STATUS": _show_parser("TABLE STATUS"),
    385            "FULL TABLES": _show_parser("TABLES", full=True),
    386            "TABLES": _show_parser("TABLES"),
    387            "TRIGGERS": _show_parser("TRIGGERS"),
    388            "GLOBAL VARIABLES": _show_parser("VARIABLES", global_=True),
    389            "SESSION VARIABLES": _show_parser("VARIABLES"),
    390            "VARIABLES": _show_parser("VARIABLES"),
    391            "WARNINGS": _show_parser("WARNINGS"),
    392        }
    393
    394        PROPERTY_PARSERS = {
    395            **parser.Parser.PROPERTY_PARSERS,
    396            "LOCK": lambda self: self._parse_property_assignment(exp.LockProperty),
    397        }
    398
    399        SET_PARSERS = {
    400            **parser.Parser.SET_PARSERS,
    401            "PERSIST": lambda self: self._parse_set_item_assignment("PERSIST"),
    402            "PERSIST_ONLY": lambda self: self._parse_set_item_assignment("PERSIST_ONLY"),
    403            "CHARACTER SET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    404            "CHARSET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
    405            "NAMES": lambda self: self._parse_set_item_names(),
    406        }
    407
    408        CONSTRAINT_PARSERS = {
    409            **parser.Parser.CONSTRAINT_PARSERS,
    410            "FULLTEXT": lambda self: self._parse_index_constraint(kind="FULLTEXT"),
    411            "INDEX": lambda self: self._parse_index_constraint(),
    412            "KEY": lambda self: self._parse_index_constraint(),
    413            "SPATIAL": lambda self: self._parse_index_constraint(kind="SPATIAL"),
    414        }
    415
    416        SCHEMA_UNNAMED_CONSTRAINTS = {
    417            *parser.Parser.SCHEMA_UNNAMED_CONSTRAINTS,
    418            "FULLTEXT",
    419            "INDEX",
    420            "KEY",
    421            "SPATIAL",
    422        }
    423
    424        PROFILE_TYPES: parser.OPTIONS_TYPE = {
    425            **dict.fromkeys(("ALL", "CPU", "IPC", "MEMORY", "SOURCE", "SWAPS"), tuple()),
    426            "BLOCK": ("IO",),
    427            "CONTEXT": ("SWITCHES",),
    428            "PAGE": ("FAULTS",),
    429        }
    430
    431        TYPE_TOKENS = {
    432            *parser.Parser.TYPE_TOKENS,
    433            TokenType.SET,
    434        }
    435
    436        ENUM_TYPE_TOKENS = {
    437            *parser.Parser.ENUM_TYPE_TOKENS,
    438            TokenType.SET,
    439        }
    440
    441        LOG_DEFAULTS_TO_LN = True
    442        STRING_ALIASES = True
    443        VALUES_FOLLOWED_BY_PAREN = False
    444
    445        def _parse_primary_key_part(self) -> t.Optional[exp.Expression]:
    446            this = self._parse_id_var()
    447            if not self._match(TokenType.L_PAREN):
    448                return this
    449
    450            expression = self._parse_number()
    451            self._match_r_paren()
    452            return self.expression(exp.ColumnPrefix, this=this, expression=expression)
    453
    454        def _parse_index_constraint(
    455            self, kind: t.Optional[str] = None
    456        ) -> exp.IndexColumnConstraint:
    457            if kind:
    458                self._match_texts(("INDEX", "KEY"))
    459
    460            this = self._parse_id_var(any_token=False)
    461            index_type = self._match(TokenType.USING) and self._advance_any() and self._prev.text
    462            schema = self._parse_schema()
    463
    464            options = []
    465            while True:
    466                if self._match_text_seq("KEY_BLOCK_SIZE"):
    467                    self._match(TokenType.EQ)
    468                    opt = exp.IndexConstraintOption(key_block_size=self._parse_number())
    469                elif self._match(TokenType.USING):
    470                    opt = exp.IndexConstraintOption(using=self._advance_any() and self._prev.text)
    471                elif self._match_text_seq("WITH", "PARSER"):
    472                    opt = exp.IndexConstraintOption(parser=self._parse_var(any_token=True))
    473                elif self._match(TokenType.COMMENT):
    474                    opt = exp.IndexConstraintOption(comment=self._parse_string())
    475                elif self._match_text_seq("VISIBLE"):
    476                    opt = exp.IndexConstraintOption(visible=True)
    477                elif self._match_text_seq("INVISIBLE"):
    478                    opt = exp.IndexConstraintOption(visible=False)
    479                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    480                    self._match(TokenType.EQ)
    481                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    482                elif self._match_text_seq("ENGINE_ATTRIBUTE"):
    483                    self._match(TokenType.EQ)
    484                    opt = exp.IndexConstraintOption(engine_attr=self._parse_string())
    485                elif self._match_text_seq("SECONDARY_ENGINE_ATTRIBUTE"):
    486                    self._match(TokenType.EQ)
    487                    opt = exp.IndexConstraintOption(secondary_engine_attr=self._parse_string())
    488                else:
    489                    opt = None
    490
    491                if not opt:
    492                    break
    493
    494                options.append(opt)
    495
    496            return self.expression(
    497                exp.IndexColumnConstraint,
    498                this=this,
    499                schema=schema,
    500                kind=kind,
    501                index_type=index_type,
    502                options=options,
    503            )
    504
    505        def _parse_show_mysql(
    506            self,
    507            this: str,
    508            target: bool | str = False,
    509            full: t.Optional[bool] = None,
    510            global_: t.Optional[bool] = None,
    511        ) -> exp.Show:
    512            if target:
    513                if isinstance(target, str):
    514                    self._match_text_seq(target)
    515                target_id = self._parse_id_var()
    516            else:
    517                target_id = None
    518
    519            log = self._parse_string() if self._match_text_seq("IN") else None
    520
    521            if this in ("BINLOG EVENTS", "RELAYLOG EVENTS"):
    522                position = self._parse_number() if self._match_text_seq("FROM") else None
    523                db = None
    524            else:
    525                position = None
    526                db = None
    527
    528                if self._match(TokenType.FROM):
    529                    db = self._parse_id_var()
    530                elif self._match(TokenType.DOT):
    531                    db = target_id
    532                    target_id = self._parse_id_var()
    533
    534            channel = self._parse_id_var() if self._match_text_seq("FOR", "CHANNEL") else None
    535
    536            like = self._parse_string() if self._match_text_seq("LIKE") else None
    537            where = self._parse_where()
    538
    539            if this == "PROFILE":
    540                types = self._parse_csv(lambda: self._parse_var_from_options(self.PROFILE_TYPES))
    541                query = self._parse_number() if self._match_text_seq("FOR", "QUERY") else None
    542                offset = self._parse_number() if self._match_text_seq("OFFSET") else None
    543                limit = self._parse_number() if self._match_text_seq("LIMIT") else None
    544            else:
    545                types, query = None, None
    546                offset, limit = self._parse_oldstyle_limit()
    547
    548            mutex = True if self._match_text_seq("MUTEX") else None
    549            mutex = False if self._match_text_seq("STATUS") else mutex
    550
    551            return self.expression(
    552                exp.Show,
    553                this=this,
    554                target=target_id,
    555                full=full,
    556                log=log,
    557                position=position,
    558                db=db,
    559                channel=channel,
    560                like=like,
    561                where=where,
    562                types=types,
    563                query=query,
    564                offset=offset,
    565                limit=limit,
    566                mutex=mutex,
    567                **{"global": global_},  # type: ignore
    568            )
    569
    570        def _parse_oldstyle_limit(
    571            self,
    572        ) -> t.Tuple[t.Optional[exp.Expression], t.Optional[exp.Expression]]:
    573            limit = None
    574            offset = None
    575            if self._match_text_seq("LIMIT"):
    576                parts = self._parse_csv(self._parse_number)
    577                if len(parts) == 1:
    578                    limit = parts[0]
    579                elif len(parts) == 2:
    580                    limit = parts[1]
    581                    offset = parts[0]
    582
    583            return offset, limit
    584
    585        def _parse_set_item_charset(self, kind: str) -> exp.Expression:
    586            this = self._parse_string() or self._parse_id_var()
    587            return self.expression(exp.SetItem, this=this, kind=kind)
    588
    589        def _parse_set_item_names(self) -> exp.Expression:
    590            charset = self._parse_string() or self._parse_id_var()
    591            if self._match_text_seq("COLLATE"):
    592                collate = self._parse_string() or self._parse_id_var()
    593            else:
    594                collate = None
    595
    596            return self.expression(exp.SetItem, this=charset, collate=collate, kind="NAMES")
    597
    598        def _parse_type(self, parse_interval: bool = True) -> t.Optional[exp.Expression]:
    599            # mysql binary is special and can work anywhere, even in order by operations
    600            # it operates like a no paren func
    601            if self._match(TokenType.BINARY, advance=False):
    602                data_type = self._parse_types(check_func=True, allow_identifiers=False)
    603
    604                if isinstance(data_type, exp.DataType):
    605                    return self.expression(exp.Cast, this=self._parse_column(), to=data_type)
    606
    607            return super()._parse_type(parse_interval=parse_interval)
    608
    609        def _parse_chr(self) -> t.Optional[exp.Expression]:
    610            expressions = self._parse_csv(self._parse_conjunction)
    611            kwargs: t.Dict[str, t.Any] = {"this": seq_get(expressions, 0)}
    612
    613            if len(expressions) > 1:
    614                kwargs["expressions"] = expressions[1:]
    615
    616            if self._match(TokenType.USING):
    617                kwargs["charset"] = self._parse_var()
    618
    619            return self.expression(exp.Chr, **kwargs)

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: The amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

FUNC_TOKENS = {<TokenType.ENUM16: 'ENUM16'>, <TokenType.ENUM: 'ENUM'>, <TokenType.XOR: 'XOR'>, <TokenType.RIGHT: 'RIGHT'>, <TokenType.TIME: 'TIME'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.IPV6: 'IPV6'>, <TokenType.VAR: 'VAR'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.NESTED: 'NESTED'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.ANY: 'ANY'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.JSONB: 'JSONB'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.INT128: 'INT128'>, <TokenType.TEXT: 'TEXT'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.JSON: 'JSON'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.RLIKE: 'RLIKE'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.INET: 'INET'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.IPV4: 'IPV4'>, <TokenType.INDEX: 'INDEX'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.UINT128: 'UINT128'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.SOME: 'SOME'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.DATE32: 'DATE32'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.MAP: 'MAP'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.MONEY: 'MONEY'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.NAME: 'NAME'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.UUID: 'UUID'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.GLOB: 'GLOB'>, <TokenType.INT256: 'INT256'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.ILIKE: 'ILIKE'>, <TokenType.YEAR: 'YEAR'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.OFFSET: 'OFFSET'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.UINT: 'UINT'>, <TokenType.UNNEST: 'UNNEST'>, <TokenType.WINDOW: 'WINDOW'>, <TokenType.NULL: 'NULL'>, <TokenType.BINARY: 'BINARY'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.ALL: 'ALL'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.VALUES: 'VALUES'>, <TokenType.FILTER: 'FILTER'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.FIRST: 'FIRST'>, <TokenType.MERGE: 'MERGE'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.BIT: 'BIT'>, <TokenType.XML: 'XML'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.RANGE: 'RANGE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.SUPER: 'SUPER'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.DATE: 'DATE'>, <TokenType.LEFT: 'LEFT'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.CHAR: 'CHAR'>, <TokenType.INSERT: 'INSERT'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.UINT256: 'UINT256'>, <TokenType.LIKE: 'LIKE'>, <TokenType.ROW: 'ROW'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.INT: 'INT'>}
CONJUNCTION = {<TokenType.AND: 'AND'>: <class 'sqlglot.expressions.And'>, <TokenType.OR: 'OR'>: <class 'sqlglot.expressions.Or'>, <TokenType.DAMP: 'DAMP'>: <class 'sqlglot.expressions.And'>, <TokenType.XOR: 'XOR'>: <class 'sqlglot.expressions.Xor'>, <TokenType.DPIPE: 'DPIPE'>: <class 'sqlglot.expressions.Or'>}
TABLE_ALIAS_TOKENS = {<TokenType.COMMIT: 'COMMIT'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.ENUM: 'ENUM'>, <TokenType.TIME: 'TIME'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.CACHE: 'CACHE'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.VAR: 'VAR'>, <TokenType.IPV6: 'IPV6'>, <TokenType.SHOW: 'SHOW'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.ASC: 'ASC'>, <TokenType.NESTED: 'NESTED'>, <TokenType.SET: 'SET'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.ANY: 'ANY'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.JSONB: 'JSONB'>, <TokenType.NEXT: 'NEXT'>, <TokenType.PROCEDURE: 'PROCEDURE'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.DESC: 'DESC'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.VIEW: 'VIEW'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.END: 'END'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.INT128: 'INT128'>, <TokenType.TEXT: 'TEXT'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.ANTI: 'ANTI'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.KILL: 'KILL'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.JSON: 'JSON'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.FALSE: 'FALSE'>, <TokenType.INET: 'INET'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.KEEP: 'KEEP'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.FINAL: 'FINAL'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.IPV4: 'IPV4'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.INDEX: 'INDEX'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.UINT128: 'UINT128'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.DELETE: 'DELETE'>, <TokenType.SOME: 'SOME'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.DATE32: 'DATE32'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.MAP: 'MAP'>, <TokenType.SEMI: 'SEMI'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.DIV: 'DIV'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.MONEY: 'MONEY'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.IS: 'IS'>, <TokenType.NAME: 'NAME'>, <TokenType.UUID: 'UUID'>, <TokenType.TOP: 'TOP'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.INT256: 'INT256'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.YEAR: 'YEAR'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.LOAD: 'LOAD'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.CASE: 'CASE'>, <TokenType.UINT: 'UINT'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.NULL: 'NULL'>, <TokenType.BINARY: 'BINARY'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.ALL: 'ALL'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.FILTER: 'FILTER'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.FIRST: 'FIRST'>, <TokenType.MERGE: 'MERGE'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.BIT: 'BIT'>, <TokenType.XML: 'XML'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.RANGE: 'RANGE'>, <TokenType.TRUE: 'TRUE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.SUPER: 'SUPER'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.DATE: 'DATE'>, <TokenType.ROWS: 'ROWS'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.CHAR: 'CHAR'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.UINT256: 'UINT256'>, <TokenType.ROW: 'ROW'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.MODEL: 'MODEL'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.INT: 'INT'>}
RANGE_PARSERS = {<TokenType.BETWEEN: 'BETWEEN'>: <function Parser.<lambda>>, <TokenType.GLOB: 'GLOB'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.ILIKE: 'ILIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IN: 'IN'>: <function Parser.<lambda>>, <TokenType.IRLIKE: 'IRLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IS: 'IS'>: <function Parser.<lambda>>, <TokenType.LIKE: 'LIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.OVERLAPS: 'OVERLAPS'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.RLIKE: 'RLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.SIMILAR_TO: 'SIMILAR_TO'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.FOR: 'FOR'>: <function Parser.<lambda>>, <TokenType.MEMBER_OF: 'MEMBER_OF'>: <function MySQL.Parser.<lambda>>}
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ADD_MONTHS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AddMonths'>>, 'ANONYMOUS_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnonymousAggFunc'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'APPROX_TOP_K': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxTopK'>>, 'ARG_MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARGMAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'MAX_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARG_MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARGMIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'MIN_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_OVERLAPS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayOverlaps'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_TO_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'ARRAY_UNIQUE_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CBRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cbrt'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CHR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'COLLATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Collate'>>, 'COMBINED_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedAggFunc'>>, 'COMBINED_PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedParameterizedAgg'>>, 'CONCAT': <function Parser.<lambda>>, 'CONCAT_WS': <function Parser.<lambda>>, 'CONNECT_BY_ROOT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConnectByRoot'>>, 'CONVERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Convert'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COUNTIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <function MySQL.Parser.<lambda>>, 'DATE_ADD': <function build_date_delta_with_interval.<locals>._builder>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <function build_date_delta_with_interval.<locals>._builder>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateTrunc'>>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <function MySQL.Parser.<lambda>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <function MySQL.Parser.<lambda>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <function MySQL.Parser.<lambda>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <function MySQL.Parser.<lambda>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ExplodeOuter'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FIRST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.First'>>, 'FIRST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FirstValue'>>, 'FLATTEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Flatten'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GENERATE_SERIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'IIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'IS_INF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'ISINF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'IS_NAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'ISNAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'J_S_O_N_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArray'>>, 'J_S_O_N_ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayAgg'>>, 'JSON_ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayContains'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <function build_extract_json_with_path.<locals>._builder>, 'JSON_EXTRACT_SCALAR': <function build_extract_json_with_path.<locals>._builder>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'J_S_O_N_OBJECT_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObjectAgg'>>, 'J_S_O_N_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONTable'>>, 'LAG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lag'>>, 'LAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Last'>>, 'LAST_DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastValue'>>, 'LEAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lead'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <function build_logarithm>, 'LOG10': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log10'>>, 'LOG2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log2'>>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MD5_DIGEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5Digest'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <function MySQL.Parser.<lambda>>, 'MONTHS_BETWEEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MonthsBetween'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NTH_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NthValue'>>, 'NULLIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nullif'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PARSE_JSON': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'JSON_PARSE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POSEXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PosexplodeOuter'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'PREDICT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Predict'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'RAND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDOM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Randn'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpExtract'>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_REPLACE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpReplace'>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SIGN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SIGNUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STARTS_WITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STARTSWITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <function _str_to_date>, 'STR_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'STUFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIMEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Timestamp'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMPDIFF': <function build_date_delta.<locals>._builder>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMPFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToArray'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToChar'>>, 'TO_DAYS': <function MySQL.Parser.<lambda>>, 'TO_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToNumber'>>, 'TRANSFORM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Transform'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsDiff'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'TS_OR_DS_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTime'>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixDate'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UPPER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function build_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <function MySQL.Parser.<lambda>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <function MySQL.Parser.<lambda>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'XOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Xor'>>, 'YEAR': <function MySQL.Parser.<lambda>>, 'GLOB': <function Parser.<lambda>>, 'JSON_EXTRACT_PATH_TEXT': <function build_extract_json_with_path.<locals>._builder>, 'LIKE': <function build_like>, 'DATE_FORMAT': <function build_formatted_time.<locals>._builder>, 'INSTR': <function MySQL.Parser.<lambda>>, 'ISNULL': <function isnull_to_is_null>, 'LOCATE': <function locate_to_strposition>, 'MAKETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'MONTHNAME': <function MySQL.Parser.<lambda>>}
FUNCTION_PARSERS = {'CAST': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'JSON_OBJECTAGG': <function Parser.<lambda>>, 'JSON_TABLE': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'PREDICT': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRIM': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>, 'CHAR': <function MySQL.Parser.<lambda>>, 'GROUP_CONCAT': <function MySQL.Parser.<lambda>>, 'VALUES': <function MySQL.Parser.<lambda>>}
STATEMENT_PARSERS = {<TokenType.ALTER: 'ALTER'>: <function Parser.<lambda>>, <TokenType.BEGIN: 'BEGIN'>: <function Parser.<lambda>>, <TokenType.CACHE: 'CACHE'>: <function Parser.<lambda>>, <TokenType.COMMIT: 'COMMIT'>: <function Parser.<lambda>>, <TokenType.COMMENT: 'COMMENT'>: <function Parser.<lambda>>, <TokenType.CREATE: 'CREATE'>: <function Parser.<lambda>>, <TokenType.DELETE: 'DELETE'>: <function Parser.<lambda>>, <TokenType.DESC: 'DESC'>: <function Parser.<lambda>>, <TokenType.DESCRIBE: 'DESCRIBE'>: <function Parser.<lambda>>, <TokenType.DROP: 'DROP'>: <function Parser.<lambda>>, <TokenType.INSERT: 'INSERT'>: <function Parser.<lambda>>, <TokenType.KILL: 'KILL'>: <function Parser.<lambda>>, <TokenType.LOAD: 'LOAD'>: <function Parser.<lambda>>, <TokenType.MERGE: 'MERGE'>: <function Parser.<lambda>>, <TokenType.PIVOT: 'PIVOT'>: <function Parser.<lambda>>, <TokenType.PRAGMA: 'PRAGMA'>: <function Parser.<lambda>>, <TokenType.REFRESH: 'REFRESH'>: <function Parser.<lambda>>, <TokenType.ROLLBACK: 'ROLLBACK'>: <function Parser.<lambda>>, <TokenType.SET: 'SET'>: <function Parser.<lambda>>, <TokenType.UNCACHE: 'UNCACHE'>: <function Parser.<lambda>>, <TokenType.UPDATE: 'UPDATE'>: <function Parser.<lambda>>, <TokenType.TRUNCATE: 'TRUNCATE'>: <function Parser.<lambda>>, <TokenType.USE: 'USE'>: <function Parser.<lambda>>, <TokenType.SHOW: 'SHOW'>: <function MySQL.Parser.<lambda>>}
SHOW_PARSERS = {'BINARY LOGS': <function _show_parser.<locals>._parse>, 'MASTER LOGS': <function _show_parser.<locals>._parse>, 'BINLOG EVENTS': <function _show_parser.<locals>._parse>, 'CHARACTER SET': <function _show_parser.<locals>._parse>, 'CHARSET': <function _show_parser.<locals>._parse>, 'COLLATION': <function _show_parser.<locals>._parse>, 'FULL COLUMNS': <function _show_parser.<locals>._parse>, 'COLUMNS': <function _show_parser.<locals>._parse>, 'CREATE DATABASE': <function _show_parser.<locals>._parse>, 'CREATE EVENT': <function _show_parser.<locals>._parse>, 'CREATE FUNCTION': <function _show_parser.<locals>._parse>, 'CREATE PROCEDURE': <function _show_parser.<locals>._parse>, 'CREATE TABLE': <function _show_parser.<locals>._parse>, 'CREATE TRIGGER': <function _show_parser.<locals>._parse>, 'CREATE VIEW': <function _show_parser.<locals>._parse>, 'DATABASES': <function _show_parser.<locals>._parse>, 'SCHEMAS': <function _show_parser.<locals>._parse>, 'ENGINE': <function _show_parser.<locals>._parse>, 'STORAGE ENGINES': <function _show_parser.<locals>._parse>, 'ENGINES': <function _show_parser.<locals>._parse>, 'ERRORS': <function _show_parser.<locals>._parse>, 'EVENTS': <function _show_parser.<locals>._parse>, 'FUNCTION CODE': <function _show_parser.<locals>._parse>, 'FUNCTION STATUS': <function _show_parser.<locals>._parse>, 'GRANTS': <function _show_parser.<locals>._parse>, 'INDEX': <function _show_parser.<locals>._parse>, 'MASTER STATUS': <function _show_parser.<locals>._parse>, 'OPEN TABLES': <function _show_parser.<locals>._parse>, 'PLUGINS': <function _show_parser.<locals>._parse>, 'PROCEDURE CODE': <function _show_parser.<locals>._parse>, 'PROCEDURE STATUS': <function _show_parser.<locals>._parse>, 'PRIVILEGES': <function _show_parser.<locals>._parse>, 'FULL PROCESSLIST': <function _show_parser.<locals>._parse>, 'PROCESSLIST': <function _show_parser.<locals>._parse>, 'PROFILE': <function _show_parser.<locals>._parse>, 'PROFILES': <function _show_parser.<locals>._parse>, 'RELAYLOG EVENTS': <function _show_parser.<locals>._parse>, 'REPLICAS': <function _show_parser.<locals>._parse>, 'SLAVE HOSTS': <function _show_parser.<locals>._parse>, 'REPLICA STATUS': <function _show_parser.<locals>._parse>, 'SLAVE STATUS': <function _show_parser.<locals>._parse>, 'GLOBAL STATUS': <function _show_parser.<locals>._parse>, 'SESSION STATUS': <function _show_parser.<locals>._parse>, 'STATUS': <function _show_parser.<locals>._parse>, 'TABLE STATUS': <function _show_parser.<locals>._parse>, 'FULL TABLES': <function _show_parser.<locals>._parse>, 'TABLES': <function _show_parser.<locals>._parse>, 'TRIGGERS': <function _show_parser.<locals>._parse>, 'GLOBAL VARIABLES': <function _show_parser.<locals>._parse>, 'SESSION VARIABLES': <function _show_parser.<locals>._parse>, 'VARIABLES': <function _show_parser.<locals>._parse>, 'WARNINGS': <function _show_parser.<locals>._parse>}
PROPERTY_PARSERS = {'ALGORITHM': <function Parser.<lambda>>, 'AUTO': <function Parser.<lambda>>, 'AUTO_INCREMENT': <function Parser.<lambda>>, 'BACKUP': <function Parser.<lambda>>, 'BLOCKCOMPRESSION': <function Parser.<lambda>>, 'CHARSET': <function Parser.<lambda>>, 'CHARACTER SET': <function Parser.<lambda>>, 'CHECKSUM': <function Parser.<lambda>>, 'CLUSTER BY': <function Parser.<lambda>>, 'CLUSTERED': <function Parser.<lambda>>, 'COLLATE': <function Parser.<lambda>>, 'COMMENT': <function Parser.<lambda>>, 'CONTAINS': <function Parser.<lambda>>, 'COPY': <function Parser.<lambda>>, 'DATABLOCKSIZE': <function Parser.<lambda>>, 'DEFINER': <function Parser.<lambda>>, 'DETERMINISTIC': <function Parser.<lambda>>, 'DISTKEY': <function Parser.<lambda>>, 'DISTSTYLE': <function Parser.<lambda>>, 'ENGINE': <function Parser.<lambda>>, 'EXECUTE': <function Parser.<lambda>>, 'EXTERNAL': <function Parser.<lambda>>, 'FALLBACK': <function Parser.<lambda>>, 'FORMAT': <function Parser.<lambda>>, 'FREESPACE': <function Parser.<lambda>>, 'GLOBAL': <function Parser.<lambda>>, 'HEAP': <function Parser.<lambda>>, 'IMMUTABLE': <function Parser.<lambda>>, 'INHERITS': <function Parser.<lambda>>, 'INPUT': <function Parser.<lambda>>, 'JOURNAL': <function Parser.<lambda>>, 'LANGUAGE': <function Parser.<lambda>>, 'LAYOUT': <function Parser.<lambda>>, 'LIFETIME': <function Parser.<lambda>>, 'LIKE': <function Parser.<lambda>>, 'LOCATION': <function Parser.<lambda>>, 'LOCK': <function MySQL.Parser.<lambda>>, 'LOCKING': <function Parser.<lambda>>, 'LOG': <function Parser.<lambda>>, 'MATERIALIZED': <function Parser.<lambda>>, 'MERGEBLOCKRATIO': <function Parser.<lambda>>, 'MODIFIES': <function Parser.<lambda>>, 'MULTISET': <function Parser.<lambda>>, 'NO': <function Parser.<lambda>>, 'ON': <function Parser.<lambda>>, 'ORDER BY': <function Parser.<lambda>>, 'OUTPUT': <function Parser.<lambda>>, 'PARTITION': <function Parser.<lambda>>, 'PARTITION BY': <function Parser.<lambda>>, 'PARTITIONED BY': <function Parser.<lambda>>, 'PARTITIONED_BY': <function Parser.<lambda>>, 'PRIMARY KEY': <function Parser.<lambda>>, 'RANGE': <function Parser.<lambda>>, 'READS': <function Parser.<lambda>>, 'REMOTE': <function Parser.<lambda>>, 'RETURNS': <function Parser.<lambda>>, 'ROW': <function Parser.<lambda>>, 'ROW_FORMAT': <function Parser.<lambda>>, 'SAMPLE': <function Parser.<lambda>>, 'SET': <function Parser.<lambda>>, 'SETTINGS': <function Parser.<lambda>>, 'SHARING': <function Parser.<lambda>>, 'SORTKEY': <function Parser.<lambda>>, 'SOURCE': <function Parser.<lambda>>, 'STABLE': <function Parser.<lambda>>, 'STORED': <function Parser.<lambda>>, 'SYSTEM_VERSIONING': <function Parser.<lambda>>, 'TBLPROPERTIES': <function Parser.<lambda>>, 'TEMP': <function Parser.<lambda>>, 'TEMPORARY': <function Parser.<lambda>>, 'TO': <function Parser.<lambda>>, 'TRANSIENT': <function Parser.<lambda>>, 'TRANSFORM': <function Parser.<lambda>>, 'TTL': <function Parser.<lambda>>, 'USING': <function Parser.<lambda>>, 'UNLOGGED': <function Parser.<lambda>>, 'VOLATILE': <function Parser.<lambda>>, 'WITH': <function Parser.<lambda>>}
SET_PARSERS = {'GLOBAL': <function Parser.<lambda>>, 'LOCAL': <function Parser.<lambda>>, 'SESSION': <function Parser.<lambda>>, 'TRANSACTION': <function Parser.<lambda>>, 'PERSIST': <function MySQL.Parser.<lambda>>, 'PERSIST_ONLY': <function MySQL.Parser.<lambda>>, 'CHARACTER SET': <function MySQL.Parser.<lambda>>, 'CHARSET': <function MySQL.Parser.<lambda>>, 'NAMES': <function MySQL.Parser.<lambda>>}
CONSTRAINT_PARSERS = {'AUTOINCREMENT': <function Parser.<lambda>>, 'AUTO_INCREMENT': <function Parser.<lambda>>, 'CASESPECIFIC': <function Parser.<lambda>>, 'CHARACTER SET': <function Parser.<lambda>>, 'CHECK': <function Parser.<lambda>>, 'COLLATE': <function Parser.<lambda>>, 'COMMENT': <function Parser.<lambda>>, 'COMPRESS': <function Parser.<lambda>>, 'CLUSTERED': <function Parser.<lambda>>, 'NONCLUSTERED': <function Parser.<lambda>>, 'DEFAULT': <function Parser.<lambda>>, 'ENCODE': <function Parser.<lambda>>, 'EXCLUDE': <function Parser.<lambda>>, 'FOREIGN KEY': <function Parser.<lambda>>, 'FORMAT': <function Parser.<lambda>>, 'GENERATED': <function Parser.<lambda>>, 'IDENTITY': <function Parser.<lambda>>, 'INLINE': <function Parser.<lambda>>, 'LIKE': <function Parser.<lambda>>, 'NOT': <function Parser.<lambda>>, 'NULL': <function Parser.<lambda>>, 'ON': <function Parser.<lambda>>, 'PATH': <function Parser.<lambda>>, 'PERIOD': <function Parser.<lambda>>, 'PRIMARY KEY': <function Parser.<lambda>>, 'REFERENCES': <function Parser.<lambda>>, 'TITLE': <function Parser.<lambda>>, 'TTL': <function Parser.<lambda>>, 'UNIQUE': <function Parser.<lambda>>, 'UPPERCASE': <function Parser.<lambda>>, 'WITH': <function Parser.<lambda>>, 'FULLTEXT': <function MySQL.Parser.<lambda>>, 'INDEX': <function MySQL.Parser.<lambda>>, 'KEY': <function MySQL.Parser.<lambda>>, 'SPATIAL': <function MySQL.Parser.<lambda>>}
SCHEMA_UNNAMED_CONSTRAINTS = {'FULLTEXT', 'SPATIAL', 'CHECK', 'INDEX', 'LIKE', 'PRIMARY KEY', 'UNIQUE', 'KEY', 'EXCLUDE', 'PERIOD', 'FOREIGN KEY'}
PROFILE_TYPES: Dict[str, Sequence[Union[Sequence[str], str]]] = {'ALL': (), 'CPU': (), 'IPC': (), 'MEMORY': (), 'SOURCE': (), 'SWAPS': (), 'BLOCK': ('IO',), 'CONTEXT': ('SWITCHES',), 'PAGE': ('FAULTS',)}
TYPE_TOKENS = {<TokenType.DATE32: 'DATE32'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.ENUM: 'ENUM'>, <TokenType.MAP: 'MAP'>, <TokenType.TIME: 'TIME'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.IPV6: 'IPV6'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.MONEY: 'MONEY'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.NESTED: 'NESTED'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.NAME: 'NAME'>, <TokenType.SET: 'SET'>, <TokenType.UUID: 'UUID'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.INT256: 'INT256'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.JSONB: 'JSONB'>, <TokenType.YEAR: 'YEAR'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.UINT: 'UINT'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.NULL: 'NULL'>, <TokenType.INT128: 'INT128'>, <TokenType.TEXT: 'TEXT'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.BINARY: 'BINARY'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.JSON: 'JSON'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.INET: 'INET'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.BIT: 'BIT'>, <TokenType.XML: 'XML'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.SUPER: 'SUPER'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.DATE: 'DATE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.IPV4: 'IPV4'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.CHAR: 'CHAR'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.UINT256: 'UINT256'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.UINT128: 'UINT128'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.INT: 'INT'>, <TokenType.BPCHAR: 'BPCHAR'>}
ENUM_TYPE_TOKENS = {<TokenType.ENUM16: 'ENUM16'>, <TokenType.ENUM: 'ENUM'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.SET: 'SET'>}
LOG_DEFAULTS_TO_LN = True
STRING_ALIASES = True
VALUES_FOLLOWED_BY_PAREN = False
SHOW_TRIE: Dict = {'BINARY': {'LOGS': {0: True}}, 'MASTER': {'LOGS': {0: True}, 'STATUS': {0: True}}, 'BINLOG': {'EVENTS': {0: True}}, 'CHARACTER': {'SET': {0: True}}, 'CHARSET': {0: True}, 'COLLATION': {0: True}, 'FULL': {'COLUMNS': {0: True}, 'PROCESSLIST': {0: True}, 'TABLES': {0: True}}, 'COLUMNS': {0: True}, 'CREATE': {'DATABASE': {0: True}, 'EVENT': {0: True}, 'FUNCTION': {0: True}, 'PROCEDURE': {0: True}, 'TABLE': {0: True}, 'TRIGGER': {0: True}, 'VIEW': {0: True}}, 'DATABASES': {0: True}, 'SCHEMAS': {0: True}, 'ENGINE': {0: True}, 'STORAGE': {'ENGINES': {0: True}}, 'ENGINES': {0: True}, 'ERRORS': {0: True}, 'EVENTS': {0: True}, 'FUNCTION': {'CODE': {0: True}, 'STATUS': {0: True}}, 'GRANTS': {0: True}, 'INDEX': {0: True}, 'OPEN': {'TABLES': {0: True}}, 'PLUGINS': {0: True}, 'PROCEDURE': {'CODE': {0: True}, 'STATUS': {0: True}}, 'PRIVILEGES': {0: True}, 'PROCESSLIST': {0: True}, 'PROFILE': {0: True}, 'PROFILES': {0: True}, 'RELAYLOG': {'EVENTS': {0: True}}, 'REPLICAS': {0: True}, 'SLAVE': {'HOSTS': {0: True}, 'STATUS': {0: True}}, 'REPLICA': {'STATUS': {0: True}}, 'GLOBAL': {'STATUS': {0: True}, 'VARIABLES': {0: True}}, 'SESSION': {'STATUS': {0: True}, 'VARIABLES': {0: True}}, 'STATUS': {0: True}, 'TABLE': {'STATUS': {0: True}}, 'TABLES': {0: True}, 'TRIGGERS': {0: True}, 'VARIABLES': {0: True}, 'WARNINGS': {0: True}}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}, 'PERSIST': {0: True}, 'PERSIST_ONLY': {0: True}, 'CHARACTER': {'SET': {0: True}}, 'CHARSET': {0: True}, 'NAMES': {0: True}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  STRUCT_TYPE_TOKENS
  NESTED_TYPE_TOKENS
  AGGREGATE_TYPE_TOKENS
  SIGNED_TO_UNSIGNED_TYPE_TOKEN
  SUBQUERY_PREDICATES
  RESERVED_TOKENS
  DB_CREATABLES
  CREATABLES
  ID_VAR_TOKENS
  INTERVAL_VARS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  EQUALITY
  COMPARISON
  BITWISE
  TERM
  FACTOR
  EXPONENT
  TIMES
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  COLUMN_OPERATORS
  EXPRESSION_PARSERS
  UNARY_PARSERS
  STRING_PARSERS
  NUMERIC_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  ALTER_PARSERS
  NO_PAREN_FUNCTION_PARSERS
  INVALID_FUNC_NAME_TOKENS
  FUNCTIONS_WITH_ALIASED_ARGS
  KEY_VALUE_DEFINITIONS
  QUERY_MODIFIER_PARSERS
  TYPE_LITERAL_PARSERS
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  CONFLICT_ACTIONS
  CREATE_SEQUENCE
  USABLES
  INSERT_ALTERNATIVES
  CLONE_KEYWORDS
  HISTORICAL_DATA_KIND
  OPCLASS_FOLLOW_KEYWORDS
  OPTYPE_FOLLOW_TOKENS
  TABLE_INDEX_HINT_TOKENS
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  JSON_KEY_VALUE_SEPARATOR_TOKENS
  FETCH_TOKENS
  ADD_CONSTRAINT_TOKENS
  DISTINCT_TOKENS
  NULL_TOKENS
  UNNEST_OFFSET_ALIAS_TOKENS
  STRICT_CAST
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN
  TABLESAMPLE_CSV
  SET_REQUIRES_ASSIGNMENT_DELIMITER
  TRIM_PATTERN_FIRST
  MODIFIERS_ATTACHED_TO_UNION
  UNION_MODIFIERS
  NO_PAREN_IF_COMMANDS
  JSON_ARROWS_REQUIRE_JSON_TYPE
  SUPPORTS_IMPLICIT_UNNEST
  error_level
  error_message_context
  max_errors
  dialect
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class MySQL.Generator(sqlglot.generator.Generator): View Source

    621    class Generator(generator.Generator):
    622        LOCKING_READS_SUPPORTED = True
    623        NULL_ORDERING_SUPPORTED = None
    624        JOIN_HINTS = False
    625        TABLE_HINTS = True
    626        DUPLICATE_KEY_UPDATE_WITH_SET = False
    627        QUERY_HINT_SEP = " "
    628        VALUES_AS_TABLE = False
    629        NVL2_SUPPORTED = False
    630        LAST_DAY_SUPPORTS_DATE_PART = False
    631        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    632        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
    633        JSON_KEY_VALUE_PAIR_SEP = ","
    634        SUPPORTS_TO_NUMBER = False
    635
    636        TRANSFORMS = {
    637            **generator.Generator.TRANSFORMS,
    638            exp.CurrentDate: no_paren_current_date_sql,
    639            exp.DateDiff: _remove_ts_or_ds_to_date(
    640                lambda self, e: self.func("DATEDIFF", e.this, e.expression), ("this", "expression")
    641            ),
    642            exp.DateAdd: _remove_ts_or_ds_to_date(_date_add_sql("ADD")),
    643            exp.DateStrToDate: datestrtodate_sql,
    644            exp.DateSub: _remove_ts_or_ds_to_date(_date_add_sql("SUB")),
    645            exp.DateTrunc: _date_trunc_sql,
    646            exp.Day: _remove_ts_or_ds_to_date(),
    647            exp.DayOfMonth: _remove_ts_or_ds_to_date(rename_func("DAYOFMONTH")),
    648            exp.DayOfWeek: _remove_ts_or_ds_to_date(rename_func("DAYOFWEEK")),
    649            exp.DayOfYear: _remove_ts_or_ds_to_date(rename_func("DAYOFYEAR")),
    650            exp.GroupConcat: lambda self,
    651            e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
    652            exp.ILike: no_ilike_sql,
    653            exp.JSONExtractScalar: arrow_json_extract_sql,
    654            exp.Max: max_or_greatest,
    655            exp.Min: min_or_least,
    656            exp.Month: _remove_ts_or_ds_to_date(),
    657            exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
    658            exp.NullSafeNEQ: lambda self, e: f"NOT {self.binary(e, '<=>')}",
    659            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
    660            exp.Pivot: no_pivot_sql,
    661            exp.Select: transforms.preprocess(
    662                [
    663                    transforms.eliminate_distinct_on,
    664                    transforms.eliminate_semi_and_anti_joins,
    665                    transforms.eliminate_qualify,
    666                    transforms.eliminate_full_outer_join,
    667                ]
    668            ),
    669            exp.StrPosition: strposition_to_locate_sql,
    670            exp.StrToDate: _str_to_date_sql,
    671            exp.StrToTime: _str_to_date_sql,
    672            exp.Stuff: rename_func("INSERT"),
    673            exp.TableSample: no_tablesample_sql,
    674            exp.TimeFromParts: rename_func("MAKETIME"),
    675            exp.TimestampAdd: date_add_interval_sql("DATE", "ADD"),
    676            exp.TimestampDiff: lambda self, e: self.func(
    677                "TIMESTAMPDIFF", e.text("unit"), e.expression, e.this
    678            ),
    679            exp.TimestampSub: date_add_interval_sql("DATE", "SUB"),
    680            exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
    681            exp.TimeStrToTime: lambda self, e: self.sql(exp.cast(e.this, "datetime", copy=True)),
    682            exp.TimeToStr: _remove_ts_or_ds_to_date(
    683                lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e))
    684            ),
    685            exp.Trim: _trim_sql,
    686            exp.TryCast: no_trycast_sql,
    687            exp.TsOrDsAdd: _date_add_sql("ADD"),
    688            exp.TsOrDsDiff: lambda self, e: self.func("DATEDIFF", e.this, e.expression),
    689            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    690            exp.Week: _remove_ts_or_ds_to_date(),
    691            exp.WeekOfYear: _remove_ts_or_ds_to_date(rename_func("WEEKOFYEAR")),
    692            exp.Year: _remove_ts_or_ds_to_date(),
    693        }
    694
    695        UNSIGNED_TYPE_MAPPING = {
    696            exp.DataType.Type.UBIGINT: "BIGINT",
    697            exp.DataType.Type.UINT: "INT",
    698            exp.DataType.Type.UMEDIUMINT: "MEDIUMINT",
    699            exp.DataType.Type.USMALLINT: "SMALLINT",
    700            exp.DataType.Type.UTINYINT: "TINYINT",
    701            exp.DataType.Type.UDECIMAL: "DECIMAL",
    702        }
    703
    704        TIMESTAMP_TYPE_MAPPING = {
    705            exp.DataType.Type.TIMESTAMP: "DATETIME",
    706            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    707            exp.DataType.Type.TIMESTAMPLTZ: "TIMESTAMP",
    708        }
    709
    710        TYPE_MAPPING = {
    711            **generator.Generator.TYPE_MAPPING,
    712            **UNSIGNED_TYPE_MAPPING,
    713            **TIMESTAMP_TYPE_MAPPING,
    714        }
    715
    716        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMTEXT)
    717        TYPE_MAPPING.pop(exp.DataType.Type.LONGTEXT)
    718        TYPE_MAPPING.pop(exp.DataType.Type.TINYTEXT)
    719        TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMBLOB)
    720        TYPE_MAPPING.pop(exp.DataType.Type.LONGBLOB)
    721        TYPE_MAPPING.pop(exp.DataType.Type.TINYBLOB)
    722
    723        PROPERTIES_LOCATION = {
    724            **generator.Generator.PROPERTIES_LOCATION,
    725            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    726            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    727        }
    728
    729        LIMIT_FETCH = "LIMIT"
    730
    731        LIMIT_ONLY_LITERALS = True
    732
    733        # MySQL doesn't support many datatypes in cast.
    734        # https://dev.mysql.com/doc/refman/8.0/en/cast-functions.html#function_cast
    735        CAST_MAPPING = {
    736            exp.DataType.Type.BIGINT: "SIGNED",
    737            exp.DataType.Type.BOOLEAN: "SIGNED",
    738            exp.DataType.Type.INT: "SIGNED",
    739            exp.DataType.Type.TEXT: "CHAR",
    740            exp.DataType.Type.UBIGINT: "UNSIGNED",
    741            exp.DataType.Type.VARCHAR: "CHAR",
    742        }
    743
    744        TIMESTAMP_FUNC_TYPES = {
    745            exp.DataType.Type.TIMESTAMPTZ,
    746            exp.DataType.Type.TIMESTAMPLTZ,
    747        }
    748
    749        def datatype_sql(self, expression: exp.DataType) -> str:
    750            # https://dev.mysql.com/doc/refman/8.0/en/numeric-type-syntax.html
    751            result = super().datatype_sql(expression)
    752            if expression.this in self.UNSIGNED_TYPE_MAPPING:
    753                result = f"{result} UNSIGNED"
    754            return result
    755
    756        def xor_sql(self, expression: exp.Xor) -> str:
    757            if expression.expressions:
    758                return self.expressions(expression, sep=" XOR ")
    759            return super().xor_sql(expression)
    760
    761        def jsonarraycontains_sql(self, expression: exp.JSONArrayContains) -> str:
    762            return f"{self.sql(expression, 'this')} MEMBER OF({self.sql(expression, 'expression')})"
    763
    764        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    765            if expression.to.this in self.TIMESTAMP_FUNC_TYPES:
    766                return self.func("TIMESTAMP", expression.this)
    767
    768            to = self.CAST_MAPPING.get(expression.to.this)
    769
    770            if to:
    771                expression.to.set("this", to)
    772            return super().cast_sql(expression)
    773
    774        def show_sql(self, expression: exp.Show) -> str:
    775            this = f" {expression.name}"
    776            full = " FULL" if expression.args.get("full") else ""
    777            global_ = " GLOBAL" if expression.args.get("global") else ""
    778
    779            target = self.sql(expression, "target")
    780            target = f" {target}" if target else ""
    781            if expression.name in ("COLUMNS", "INDEX"):
    782                target = f" FROM{target}"
    783            elif expression.name == "GRANTS":
    784                target = f" FOR{target}"
    785
    786            db = self._prefixed_sql("FROM", expression, "db")
    787
    788            like = self._prefixed_sql("LIKE", expression, "like")
    789            where = self.sql(expression, "where")
    790
    791            types = self.expressions(expression, key="types")
    792            types = f" {types}" if types else types
    793            query = self._prefixed_sql("FOR QUERY", expression, "query")
    794
    795            if expression.name == "PROFILE":
    796                offset = self._prefixed_sql("OFFSET", expression, "offset")
    797                limit = self._prefixed_sql("LIMIT", expression, "limit")
    798            else:
    799                offset = ""
    800                limit = self._oldstyle_limit_sql(expression)
    801
    802            log = self._prefixed_sql("IN", expression, "log")
    803            position = self._prefixed_sql("FROM", expression, "position")
    804
    805            channel = self._prefixed_sql("FOR CHANNEL", expression, "channel")
    806
    807            if expression.name == "ENGINE":
    808                mutex_or_status = " MUTEX" if expression.args.get("mutex") else " STATUS"
    809            else:
    810                mutex_or_status = ""
    811
    812            return f"SHOW{full}{global_}{this}{target}{types}{db}{query}{log}{position}{channel}{mutex_or_status}{like}{where}{offset}{limit}"
    813
    814        def altercolumn_sql(self, expression: exp.AlterColumn) -> str:
    815            dtype = self.sql(expression, "dtype")
    816            if not dtype:
    817                return super().altercolumn_sql(expression)
    818
    819            this = self.sql(expression, "this")
    820            return f"MODIFY COLUMN {this} {dtype}"
    821
    822        def _prefixed_sql(self, prefix: str, expression: exp.Expression, arg: str) -> str:
    823            sql = self.sql(expression, arg)
    824            return f" {prefix} {sql}" if sql else ""
    825
    826        def _oldstyle_limit_sql(self, expression: exp.Show) -> str:
    827            limit = self.sql(expression, "limit")
    828            offset = self.sql(expression, "offset")
    829            if limit:
    830                limit_offset = f"{offset}, {limit}" if offset else limit
    831                return f" LIMIT {limit_offset}"
    832            return ""
    833
    834        def chr_sql(self, expression: exp.Chr) -> str:
    835            this = self.expressions(sqls=[expression.this] + expression.expressions)
    836            charset = expression.args.get("charset")
    837            using = f" USING {self.sql(charset)}" if charset else ""
    838            return f"CHAR({this}{using})"

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether to normalize identifiers to lowercase. Default: False.
- pad: The pad size in a formatted string. Default: 2.
- indent: The indentation size in a formatted string. Default: 2.
- normalize_functions: How to normalize function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Whether the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether to preserve comments in the output SQL code. Default: True

LOCKING_READS_SUPPORTED = True
NULL_ORDERING_SUPPORTED = None
JOIN_HINTS = False
TABLE_HINTS = True
DUPLICATE_KEY_UPDATE_WITH_SET = False
QUERY_HINT_SEP = ' '
VALUES_AS_TABLE = False
NVL2_SUPPORTED = False
LAST_DAY_SUPPORTS_DATE_PART = False
JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
JSON_PATH_BRACKETED_KEY_SUPPORTED = False
JSON_KEY_VALUE_PAIR_SEP = ','
SUPPORTS_TO_NUMBER = False
TRANSFORMS = {<class 'sqlglot.expressions.JSONPathFilter'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathKey'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRecursive'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRoot'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathScript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSelector'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSlice'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSubscript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathUnion'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathWildcard'>: <function <lambda>>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.BackupProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateAdd'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExcludeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.GlobalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.HeapProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InheritsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IntervalSpan'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.JSONExtract'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.JSONExtractScalar'>: <function arrow_json_extract_sql>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NonClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NotForReplicationColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OutputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SampleProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetConfigProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SharingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.Timestamp'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransformModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UnloggedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithOperator'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CurrentDate'>: <function no_paren_current_date_sql>, <class 'sqlglot.expressions.DateDiff'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DateSub'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DateTrunc'>: <function _date_trunc_sql>, <class 'sqlglot.expressions.Day'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DayOfMonth'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DayOfWeek'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.DayOfYear'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.GroupConcat'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.ILike'>: <function no_ilike_sql>, <class 'sqlglot.expressions.Max'>: <function max_or_greatest>, <class 'sqlglot.expressions.Min'>: <function min_or_least>, <class 'sqlglot.expressions.Month'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.NullSafeEQ'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.NullSafeNEQ'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.ParseJSON'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.StrPosition'>: <function strposition_to_locate_sql>, <class 'sqlglot.expressions.StrToDate'>: <function _str_to_date_sql>, <class 'sqlglot.expressions.StrToTime'>: <function _str_to_date_sql>, <class 'sqlglot.expressions.Stuff'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TableSample'>: <function no_tablesample_sql>, <class 'sqlglot.expressions.TimeFromParts'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampAdd'>: <function date_add_interval_sql.<locals>.func>, <class 'sqlglot.expressions.TimestampDiff'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.TimestampSub'>: <function date_add_interval_sql.<locals>.func>, <class 'sqlglot.expressions.TimeStrToUnix'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeStrToTime'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.Trim'>: <function _trim_sql>, <class 'sqlglot.expressions.TryCast'>: <function no_trycast_sql>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.TsOrDsDiff'>: <function MySQL.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function _ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.Week'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.WeekOfYear'>: <function _remove_ts_or_ds_to_date.<locals>.func>, <class 'sqlglot.expressions.Year'>: <function _remove_ts_or_ds_to_date.<locals>.func>}
UNSIGNED_TYPE_MAPPING = {<Type.UBIGINT: 'UBIGINT'>: 'BIGINT', <Type.UINT: 'UINT'>: 'INT', <Type.UMEDIUMINT: 'UMEDIUMINT'>: 'MEDIUMINT', <Type.USMALLINT: 'USMALLINT'>: 'SMALLINT', <Type.UTINYINT: 'UTINYINT'>: 'TINYINT', <Type.UDECIMAL: 'UDECIMAL'>: 'DECIMAL'}
TIMESTAMP_TYPE_MAPPING = {<Type.TIMESTAMP: 'TIMESTAMP'>: 'DATETIME', <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>: 'TIMESTAMP', <Type.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>: 'TIMESTAMP'}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.INET: 'INET'>: 'INET', <Type.UBIGINT: 'UBIGINT'>: 'BIGINT', <Type.UINT: 'UINT'>: 'INT', <Type.UMEDIUMINT: 'UMEDIUMINT'>: 'MEDIUMINT', <Type.USMALLINT: 'USMALLINT'>: 'SMALLINT', <Type.UTINYINT: 'UTINYINT'>: 'TINYINT', <Type.UDECIMAL: 'UDECIMAL'>: 'DECIMAL', <Type.TIMESTAMP: 'TIMESTAMP'>: 'DATETIME', <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>: 'TIMESTAMP', <Type.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>: 'TIMESTAMP'}
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BackupProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.GlobalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.HeapProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.InheritsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.InputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OutputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PartitionedOfProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SampleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SetConfigProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SharingProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SequenceProperties'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.TransformModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.UnloggedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.WithSystemVersioningProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>}
LIMIT_FETCH = 'LIMIT'
LIMIT_ONLY_LITERALS = True
CAST_MAPPING = {<Type.BIGINT: 'BIGINT'>: 'SIGNED', <Type.BOOLEAN: 'BOOLEAN'>: 'SIGNED', <Type.INT: 'INT'>: 'SIGNED', <Type.TEXT: 'TEXT'>: 'CHAR', <Type.UBIGINT: 'UBIGINT'>: 'UNSIGNED', <Type.VARCHAR: 'VARCHAR'>: 'CHAR'}
TIMESTAMP_FUNC_TYPES = {<Type.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>}
def datatype_sql(self, expression: sqlglot.expressions.DataType) -> str: View Source

    749        def datatype_sql(self, expression: exp.DataType) -> str:
    750            # https://dev.mysql.com/doc/refman/8.0/en/numeric-type-syntax.html
    751            result = super().datatype_sql(expression)
    752            if expression.this in self.UNSIGNED_TYPE_MAPPING:
    753                result = f"{result} UNSIGNED"
    754            return result

def xor_sql(self, expression: sqlglot.expressions.Xor) -> str: View Source

    756        def xor_sql(self, expression: exp.Xor) -> str:
    757            if expression.expressions:
    758                return self.expressions(expression, sep=" XOR ")
    759            return super().xor_sql(expression)

def jsonarraycontains_sql(self, expression: sqlglot.expressions.JSONArrayContains) -> str: View Source

    761        def jsonarraycontains_sql(self, expression: exp.JSONArrayContains) -> str:
    762            return f"{self.sql(expression, 'this')} MEMBER OF({self.sql(expression, 'expression')})"

def cast_sql( self, expression: sqlglot.expressions.Cast, safe_prefix: Optional[str] = None) -> str: View Source

    764        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    765            if expression.to.this in self.TIMESTAMP_FUNC_TYPES:
    766                return self.func("TIMESTAMP", expression.this)
    767
    768            to = self.CAST_MAPPING.get(expression.to.this)
    769
    770            if to:
    771                expression.to.set("this", to)
    772            return super().cast_sql(expression)

def show_sql(self, expression: sqlglot.expressions.Show) -> str: View Source

    774        def show_sql(self, expression: exp.Show) -> str:
    775            this = f" {expression.name}"
    776            full = " FULL" if expression.args.get("full") else ""
    777            global_ = " GLOBAL" if expression.args.get("global") else ""
    778
    779            target = self.sql(expression, "target")
    780            target = f" {target}" if target else ""
    781            if expression.name in ("COLUMNS", "INDEX"):
    782                target = f" FROM{target}"
    783            elif expression.name == "GRANTS":
    784                target = f" FOR{target}"
    785
    786            db = self._prefixed_sql("FROM", expression, "db")
    787
    788            like = self._prefixed_sql("LIKE", expression, "like")
    789            where = self.sql(expression, "where")
    790
    791            types = self.expressions(expression, key="types")
    792            types = f" {types}" if types else types
    793            query = self._prefixed_sql("FOR QUERY", expression, "query")
    794
    795            if expression.name == "PROFILE":
    796                offset = self._prefixed_sql("OFFSET", expression, "offset")
    797                limit = self._prefixed_sql("LIMIT", expression, "limit")
    798            else:
    799                offset = ""
    800                limit = self._oldstyle_limit_sql(expression)
    801
    802            log = self._prefixed_sql("IN", expression, "log")
    803            position = self._prefixed_sql("FROM", expression, "position")
    804
    805            channel = self._prefixed_sql("FOR CHANNEL", expression, "channel")
    806
    807            if expression.name == "ENGINE":
    808                mutex_or_status = " MUTEX" if expression.args.get("mutex") else " STATUS"
    809            else:
    810                mutex_or_status = ""
    811
    812            return f"SHOW{full}{global_}{this}{target}{types}{db}{query}{log}{position}{channel}{mutex_or_status}{like}{where}{offset}{limit}"

def altercolumn_sql(self, expression: sqlglot.expressions.AlterColumn) -> str: View Source

    814        def altercolumn_sql(self, expression: exp.AlterColumn) -> str:
    815            dtype = self.sql(expression, "dtype")
    816            if not dtype:
    817                return super().altercolumn_sql(expression)
    818
    819            this = self.sql(expression, "this")
    820            return f"MODIFY COLUMN {this} {dtype}"

def chr_sql(self, expression: sqlglot.expressions.Chr) -> str: View Source

    834        def chr_sql(self, expression: exp.Chr) -> str:
    835            this = self.expressions(sqls=[expression.this] + expression.expressions)
    836            charset = expression.args.get("charset")
    837            using = f" USING {self.sql(charset)}" if charset else ""
    838            return f"CHAR({this}{using})"

SELECT_KINDS: Tuple[str, ...] = ()
AFTER_HAVING_MODIFIER_TRANSFORMS = {'qualify': <function Generator.<lambda>>, 'windows': <function Generator.<lambda>>}

##### Inherited Members

sqlglot.generator.Generator
  Generator
  IGNORE_NULLS_IN_FUNC
  EXPLICIT_UNION
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  SINGLE_STRING_INTERVAL
  INTERVAL_ALLOWS_PLURAL_FORM
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  QUERY_HINTS
  IS_BOOL_ALLOWED
  LIMIT_IS_TOP
  RETURNING_END
  COLUMN_JOIN_MARKS_SUPPORTED
  EXTRACT_ALLOWS_QUOTES
  TZ_TO_WITH_TIME_ZONE
  ALTER_TABLE_INCLUDE_COLUMN_KEYWORD
  UNNEST_WITH_ORDINALITY
  AGGREGATE_FILTER_SUPPORTED
  SEMI_ANTI_JOIN_WITH_SIDE
  COMPUTED_COLUMN_WITH_TYPE
  SUPPORTS_TABLE_COPY
  TABLESAMPLE_REQUIRES_PARENS
  TABLESAMPLE_SIZE_IS_ROWS
  TABLESAMPLE_KEYWORDS
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SEED_KEYWORD
  COLLATE_IS_FUNC
  DATA_TYPE_SPECIFIERS_ALLOWED
  ENSURE_BOOLS
  CTE_RECURSIVE_KEYWORD_REQUIRED
  SUPPORTS_SINGLE_ARG_CONCAT
  SUPPORTS_TABLE_ALIAS_COLUMNS
  UNPIVOT_ALIASES_ARE_IDENTIFIERS
  INSERT_OVERWRITE
  SUPPORTS_SELECT_INTO
  SUPPORTS_UNLOGGED_TABLES
  SUPPORTS_CREATE_TABLE_LIKE
  LIKE_PROPERTY_INSIDE_SCHEMA
  MULTI_ARG_DISTINCT
  JSON_PATH_SINGLE_QUOTE_ESCAPE
  SUPPORTED_JSON_PATH_PARTS
  CAN_IMPLEMENT_ARRAY_ANY
  STAR_MAPPING
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  STRUCT_DELIMITER
  PARAMETER_TOKEN
  NAMED_PLACEHOLDER_TOKEN
  RESERVED_KEYWORDS
  WITH_SEPARATED_COMMENTS
  EXCLUDE_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  PARAMETERIZABLE_TEXT_TYPES
  EXPRESSIONS_WITHOUT_NESTED_CTES
  SENTINEL_LINE_BREAK
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  dialect
  normalize_functions
  unsupported_messages
  generate
  preprocess
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  computedcolumnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  generatedasrowcolumnconstraint_sql
  periodforsystemtimeconstraint_sql
  notnullcolumnconstraint_sql
  transformcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  create_sql
  sequenceproperties_sql
  clone_sql
  describe_sql
  heredoc_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  unicodestring_sql
  rawstring_sql
  datatypeparam_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  indexparameters_sql
  index_sql
  identifier_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_name
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  partitionboundspec_sql
  partitionedofproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  withsystemversioningproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  kill_sql
  pseudotype_sql
  objectidentifier_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  historicaldata_sql
  table_parts
  table_sql
  tablesample_sql
  pivot_sql
  version_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  connect_sql
  prior_sql
  join_sql
  lambda_sql
  lateral_op
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  withfill_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognize_sql
  query_modifiers
  queryoption_sql
  offset_limit_modifiers
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  set_operations
  union_sql
  union_op
  unnest_sql
  prewhere_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_sql
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  convert_concat_args
  concat_sql
  concatws_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonpath_sql
  json_path_part
  formatjson_sql
  jsonobject_sql
  jsonobjectagg_sql
  jsonarray_sql
  jsonarrayagg_sql
  jsoncolumndef_sql
  jsonschema_sql
  jsontable_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  interval_sql
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  pivotalias_sql
  aliases_sql
  atindex_sql
  attimezone_sql
  fromtimezone_sql
  add_sql
  and_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  currentdate_sql
  currenttimestamp_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  transaction_sql
  commit_sql
  rollback_sql
  renametable_sql
  renamecolumn_sql
  altertable_sql
  add_column_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  havingmax_sql
  intdiv_sql
  dpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  propertyeq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  or_sql
  slice_sql
  sub_sql
  trycast_sql
  log_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  text_width
  format_time
  expressions
  op_expressions
  naked_property
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  tonumber_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql
  querytransform_sql
  indexconstraintoption_sql
  checkcolumnconstraint_sql
  indexcolumnconstraint_sql
  nvl2_sql
  comprehension_sql
  columnprefix_sql
  opclass_sql
  predict_sql
  forin_sql
  refresh_sql
  operator_sql
  toarray_sql
  tsordstotime_sql
  tsordstodate_sql
  unixdate_sql
  lastday_sql
  arrayany_sql
  generateseries_sql
  struct_sql
  partitionrange_sql
  truncatetable_sql
  convert_sql